from datetime import datetime
import json
import os
import threading
import time
import re
import xml.sax.saxutils
from abc import ABC, abstractmethod
from typing import List, Dict, Any, Union, Optional, Tuple, Type, Generator, Iterator
from rich.console import Console
from rich.panel import Panel
from loguru import logger
import byzerllm
from pydantic import BaseModel

from autocoder.common import AutoCoderArgs, git_utils, SourceCodeList, SourceCode
from autocoder.rag.variable_holder import VariableHolder
from autocoder.utils import llms as llms_utils
from autocoder.common.global_cancel import global_cancel
from autocoder.common import detect_env
from autocoder.common import shells
from autocoder.common.printer import Printer
from autocoder.utils.auto_project_type import ProjectTypeAnalyzer
from autocoder.common.mcp_tools.server import get_mcp_server, McpServerInfoRequest
from autocoder.common.file_monitor.monitor import FileMonitor
from autocoder.common.rulefiles import get_required_and_index_rules, get_rules
from autocoder.events.event_manager_singleton import get_event_manager
from autocoder.events.event_types import Event, EventType, EventMetadata
from autocoder.events import event_content as EventContentCreator
from autocoder.memory.active_context_manager import ActiveContextManager
from autocoder.common.action_yml_file_manager import ActionYmlFileManager
from autocoder.common.file_checkpoint.models import FileChange as CheckpointFileChange
from autocoder.common.file_checkpoint.manager import FileChangeManager as CheckpointFileChangeManager
from autocoder.linters.normal_linter import NormalLinter
from autocoder.compilers.normal_compiler import NormalCompiler

from .types import (
    BaseTool, ToolResult, AgentRequest, FileChangeEntry,
    LLMOutputEvent, LLMThinkingEvent, ToolCallEvent, ToolResultEvent, 
    CompletionEvent, ErrorEvent, TokenUsageEvent, AttemptCompletionTool,    
    PlanModeRespondTool, Message, ReplyDecision, PlanModeRespondEvent, RetryEvent,
    AgenticEditConversationConfig, ConversationAction
)
from .tool_registry import ToolRegistry
from .tools.base_tool_resolver import BaseToolResolver
from .agent_hub import AgentHub, Group, GroupMembership
from .utils import GroupUtils,GroupMemberResponse
from .default_tools import register_default_tools
from .agentic_tool_display import get_tool_display_message
from autocoder.common.utils_code_auto_generate import stream_chat_with_continue
from autocoder.common.save_formatted_log import save_formatted_log
from . import agentic_lang
from autocoder.common.autocoderargs_parser import AutoCoderArgsParser
import importlib.resources as resources

def load_tokenizer():
    from autocoder.rag.variable_holder import VariableHolder
    from tokenizers import Tokenizer

    try:
        tokenizer_path = str(resources.files("autocoder") / "data" / "tokenizer.json")
        VariableHolder.TOKENIZER_PATH = tokenizer_path
        VariableHolder.TOKENIZER_MODEL = Tokenizer.from_file(tokenizer_path)
    except FileNotFoundError:
        tokenizer_path = None

class BaseAgent(ABC):
    """
    基础代理类，所有的代理实现都应继承此类
    遵循初始化顺序规则，避免FileMonitor、token计数器等组件冲突
    """
    
    def __init__(
        self,
        name:str,
        llm: Union[byzerllm.ByzerLLM, byzerllm.SimpleByzerLLM],
        files: SourceCodeList,
        args: AutoCoderArgs,
        conversation_history: Optional[List[Dict[str, Any]]] = None,
        default_tools_list: Optional[List[str]] = None,
        custom_system_prompt: Optional[str] = None,
        conversation_config: Optional['AgenticEditConversationConfig'] = None,
        cancel_token: Optional[str] = None
    ):
        # 并行执行相关初始化
        self._parallel_executor = None
        if getattr(args, 'enable_parallel_tools', False):
            from concurrent.futures import ThreadPoolExecutor
            self._parallel_executor = ThreadPoolExecutor(max_workers=4)
        """
        初始化代理

        Args:
            name: 代理名称
            llm: 语言模型客户端
            files: 源码文件列表
            args: 配置参数
            conversation_history: 对话历史记录
            default_tools_list: 默认工具列表
            custom_system_prompt: 自定义系统提示
            conversation_config: 对话配置
            cancel_token: 取消令牌
        """
        # 1. 初始化FileMonitor（必须最先进行）
        try:
            monitor = FileMonitor(args.source_dir)
            if not monitor.is_running():
                monitor.start()
                logger.info(f"文件监控已启动: {args.source_dir}")
            else:
                logger.info(f"文件监控已在运行中: {monitor.root_dir}")
            
            # 2. 加载规则文件
            _ = get_rules(args.source_dir)
        except Exception as e:
            logger.error(f"初始化文件监控出错: {e}")

        # 3. 加载tokenizer (必须在前两步之后)
        if VariableHolder.TOKENIZER_PATH is None:
            load_tokenizer()
        
        # 4. 初始化基本组件
        self.llm = llm
        self.args = args
        self.files = files
        self.printer = Printer()
        # 对话历史存储优化：限制最大长度，实现LRU缓存
        self.max_conversation_history = getattr(args, 'max_conversation_history', 20)
        self.conversation_history = conversation_history[-self.max_conversation_history:] if conversation_history else []
        self.conversation_config = conversation_config
        self.cancel_token = cancel_token
        
        # 内存监控
        self._memory_warning_issued = False
        
        # Initialize AutoCoderArgs parser for flexible parameter parsing
        self.args_parser = AutoCoderArgsParser()
               
        
        # 5. 初始化其他组件
        self.project_type_analyzer = ProjectTypeAnalyzer(args=args, llm=self.llm)        
       # self.shadow_manager = ShadowManager(
        #     args.source_dir, args.event_file, args.ignore_clean_shadows)
        self.shadow_manager = None
        # self.shadow_linter = ShadowLinter(self.shadow_manager, verbose=False)
        self.shadow_compiler = None
        # self.shadow_compiler = ShadowCompiler(self.shadow_manager, verbose=False)
        self.shadow_linter = None

        self.checkpoint_manager = CheckpointFileChangeManager(
            project_dir=args.source_dir,
            backup_dir=os.path.join(args.source_dir,".auto-coder","checkpoint"),
            store_dir=os.path.join(args.source_dir,".auto-coder","checkpoint_store"),
            max_history=50)
        self.linter = NormalLinter(args.source_dir,verbose=False)
        self.compiler = NormalCompiler(args.source_dir,verbose=False) 
        
        
        
        # MCP 服务信息
        self.mcp_server_info = ""
        try:
            self.mcp_server = get_mcp_server()
            mcp_server_info_response = self.mcp_server.send_request(
                McpServerInfoRequest(
                    model=args.inference_model or args.model,
                    product_mode=args.product_mode,
                )
            )
            self.mcp_server_info = mcp_server_info_response.result
        except Exception as e:
            logger.error(f"Error getting MCP server info: {str(e)}")
            
        # 变更跟踪信息
        # 格式: { file_path: FileChangeEntry(...) }
        self.file_changes: Dict[str, FileChangeEntry] = {}

        self.name = name        

        # 初始化群聊/私聊功能
        self.joined_groups: Dict[str, Group] = {}
        self.private_chats: Dict[str, List[Message]] = {}        
        self.agentic_conversations: List[Dict[str,Any]] = []
        self.custom_system_prompt = custom_system_prompt or "You are a highly skilled software engineer with extensive knowledge in many programming languages, frameworks, design patterns, and best practices."
        self.refuse_reply_reason = ""
        self._group_lock = threading.RLock()  # 保护 joined_groups
        self._chat_lock = threading.RLock()   # 保护 private_chats
        # 自动注册到AgentHub
        AgentHub.register_agent(self)
        register_default_tools(params=self._render_context(), default_tools_list=default_tools_list)

    def _get_parsed_safe_zone_tokens(self) -> int:
        """
        解析 conversation_prune_safe_zone_tokens 参数，支持多种格式
        
        Returns:
            解析后的 token 数量
        """
        return self.args_parser.parse_conversation_prune_safe_zone_tokens(
            self.args.conversation_prune_safe_zone_tokens,
            self.args.code_model
        )

    def who_am_i(self, role: str) -> 'BaseAgent':
        self.custom_system_prompt = role
        return self

    def when_to_refuse_reply(self, reason: str) -> 'BaseAgent':
        self.refuse_reply_reason = reason
        return self
    
    def introduce_myself(self) -> str:        
        return self.custom_system_prompt

        
    def join_group(self, group: Group) -> 'GroupMembership':
        if group.name not in self.joined_groups:
            self.joined_groups[group.name] = group            
            group.add_member(self)            
        return GroupMembership(self, group)    
    
    def talk_to_group(self, group: Group, content: str, mentions: List['BaseAgent'] = [], print_conversation: bool = False):
        message = Message(
            sender=self.name,
            content=content,
            is_group=True,
            group_name=group.name,
            mentions=[m.name for m in mentions]
        )
        group.broadcast(message, print_conversation)
        return self

    def choose_group(self,content:str)->List[GroupMemberResponse]:
        group_utils = GroupUtils(self.llm)
        v = group_utils.auto_select_group(content,self.joined_groups.values())
        return v
            
    def talk_to(self, other: Union['BaseAgent', Group], content: str, mentions: List['BaseAgent'] = [], print_conversation: bool = False):
        if isinstance(other, Group):
            return self.talk_to_group(other, content, mentions, print_conversation)
        
        message = Message(
            sender=self.name,
            content=content,
            is_group=False,
            mentions=[m.name for m in mentions]
        )
        if print_conversation:
            print(f"[Private Chat] {self.name} -> {other.name}: {content}")
            
        # 存储双向对话记录
        self._add_private_message(other.name, message)
        other._add_private_message(self.name, message)                
        
        response = other.generate_reply(message)                                
        
        if print_conversation:
            print(f">>> {other.name} reply to {self.name} with strategy: {response.strategy}, reason: {response.reason}")

        if response.strategy == "ignore":            
            return
        elif response.strategy == "private":            
            mentions = [AgentHub.get_agent(m) for m in response.mentions]             
            other.talk_to(other=self, content=response.content, mentions=mentions, print_conversation=print_conversation)
        elif response.strategy == "broadcast":
            warning_msg = f"invalid strategy broadcast action in private chat:[{self.name}] 广播消息给群组 {other.name}: {response.content}"
            logger.warning(warning_msg)
            if print_conversation:
                print(f"[Private Chat Warning] {warning_msg}")
        return self
            
    
    def _add_private_message(self, other_name: str, message: Message):
        with self._chat_lock:
            if other_name not in self.private_chats:
                self.private_chats[other_name] = []
            self.private_chats[other_name].append(message)
    
    def threadsafe_receive(self, message: Message,print_conversation: bool = False):                
        self.receive_message(message,print_conversation=print_conversation)
        
    def receive_message(self, message: Message,print_conversation: bool = False):
        if message.is_group:
            prefix = f"[Group {message.group_name}]"
            if message.mentions and self.name in message.mentions:
                prefix += " @You"
            print(f"{prefix} {message.sender}: {message.content}")                        
            reply_decision = self.generate_reply(message)            
            print(f">>> {self.name} reply to {message.sender} with strategy: {reply_decision.strategy}, reason: {reply_decision.reason}")
            if reply_decision.strategy == "ignore":                
                return
            elif reply_decision.strategy == "private":                
                self.talk_to(other=AgentHub.get_agent(message.sender), content=reply_decision.content,print_conversation=print_conversation)
            elif reply_decision.strategy == "broadcast":
                self.joined_groups[message.group_name].broadcast(Message(
                    sender=self.name,
                    content=reply_decision.content,
                    is_group=True,
                    group_name=message.group_name,
                    mentions=reply_decision.mentions,
                    priority=reply_decision.priority
                ))
        else:
            print(f"[Private] {message.sender}: {message.content}")


    def generate_reply(self, message: Message) -> ReplyDecision:        
        user_input = self._generate_reply.prompt(message)
        events = self.agentic_run(AgentRequest(user_input=user_input))
        for event in events:
            if isinstance(event, CompletionEvent):
                from byzerllm.utils.str2model import to_model
                return to_model(ReplyDecision, event.result)
            elif isinstance(event, ErrorEvent):
                logger.error(f"Error generating reply: {event.error}")
                return ReplyDecision(strategy="ignore", content="", reason="Error generating reply")
        return None
    
    @byzerllm.prompt()
    def _generate_reply(self, message: Message) -> str:
        """
        你的名字是 {{ name }}
        {% if message.is_group %}
        当前群组是 {{ message.group_name }}
        {% endif %}
        当前时间: {{ time }}
        {% if role %}
        你对自己的描述是: 
        <who_are_you>
        {{ role }}
        </who_are_you>
        {% endif %}

        
        {{ message.sender }} 发送了一条{% if message.is_group %}群组消息{% else %}私聊消息{% endif %}：
        <message>
        {{ message.content }}
        </message>
        
        {% if message.mentions and message.mentions|length > 0 %}
        这条消息的发送者特别 @ 了用户：（{{ message.mentions|join(',') }}）。        
        {% endif %}
                
        {% if message.is_group %}
        群组对话上下文：
        <group_message_history>
        {% for msg in group_message_history %}
        {% if msg.sender == name %}
        <role>你/You</role>: <msg>{{ msg.content }}</msg>
        {% else %}
        <role>{{ msg.sender }}</role>: <msg>{{ msg.content }}</msg>
        {% endif %}
        {% endfor %}
        </group_message_history>
        {% else %}
        私聊对话上下文：
        <private_message_history>
        {% for msg in private_message_history %}
        {% if msg.sender == name %}
        <role>你/You</role>: <msg>{{ msg.content }}</msg>
        {% else %}
        <role>{{ msg.sender }}</role>: <msg>{{ msg.content }}</msg>
        {% endif %}
        {% endfor %}
        </private_message_history>
        {% endif %}

        请根据上面内容进行解答，在最后请务必使用 attempt_completion 工具，确保里面的 result 字段包含如下 Json 格式内容：

        ```json
        {
            "content": "回复内容",
            "strategy": "broadcast|private|ignore",
            "mentions": ["被提及的agent名称"],
            "priority": 优先级 0-100,
            "reason": "选择策略的原因"
        }
        ```
        ignore 表示用户发送的消息可以不进行回复,private 我们要回复用户，并且只回复给发送信息的用户，broadcast 表示要回复消息，并且回复给群组所有成员。
        *** 注意，阅读上面的所有内容，尤其关注 {% if message.is_group %}群组上下文{% else %}私聊上下文{% endif %}，判断是否使用 ignore 策略结束对话，避免无意义对话。一般在群组对话中，你没有被 @ 就无需回答，直接使用ignore策略结束对话。 ***
        {% if refuse_reply_reason %}
        当满足以下描述时，你应当拒绝回复：
        <when_to_refuse_reply>
        {{ refuse_reply_reason }}
        </when_to_refuse_reply>
        {% endif %}        
        """
        context = {
            "name": self.name,
            "role": self.custom_system_prompt,
            "time": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            "refuse_reply_reason": self.refuse_reply_reason
        }
        if message.is_group:
            group = self.joined_groups[message.group_name]
            context["group_message_history"] = group.history
        else:
            context["private_message_history"] = self.private_chats.get(message.sender, [])
            
        return context    
    
    def record_file_change(self, file_path: str, change_type: str, diff: Optional[str] = None, content: Optional[str] = None):
        """
        记录单个文件的变更信息。

        Args:
            file_path: 相对路径
            change_type: 'added' 或 'modified'
            diff: 对于 replace_in_file，传入 diff 内容
            content: 最新文件内容（可选，通常用于 write_to_file）
        """
        entry = self.file_changes.get(file_path)
        if entry is None:
            entry = FileChangeEntry(
                type=change_type, diffs=[], content=content)
            self.file_changes[file_path] = entry
        else:
            # 文件已经存在，可能之前是 added，现在又被 modified，或者多次 modified
            # 简单起见，type 用 added 优先，否则为 modified
            if entry.type != "added":
                entry.type = change_type

            # content 以最新为准
            if content is not None:
                entry.content = content

        if diff:
            entry.diffs.append(diff)
    
    def _get_all_file_changes(self) -> Dict[str, FileChangeEntry]:
        """
        获取当前记录的所有文件变更信息。

        Returns:
            字典，key 为文件路径，value 为变更详情
        """
        return self.file_changes
    
    def get_all_file_changes(self) -> Dict[str, FileChangeEntry]:
        """
        获取当前记录的所有文件变更信息。

        Returns:
            字典，key 为文件路径，value 为变更详情
        """
        return self.file_changes    
    
    
    # 缓存工具标签查找结果
    _tool_tag_cache = {}

    def _reconstruct_tool_xml(self, tool: BaseTool) -> str:
        """
        Reconstructs the XML representation of a tool call from its Pydantic model.
        """
        tool_type = type(tool)
        # 首先检查缓存
        tool_tag = self._tool_tag_cache.get(tool_type)
        if tool_tag is None:
            tool_tag = next(
                (tag for tag, model in ToolRegistry.get_tag_model_map().items() if tool_type is model), None)
            if tool_tag:
                self._tool_tag_cache[tool_type] = tool_tag
            else:
                logger.error(
                    f"Cannot find tag name for tool type {tool_type.__name__}")
                return f"<error>Could not find tag for tool {tool_type.__name__}</error>"

        xml_parts = [f"<{tool_tag}>"]
        for field_name, field_value in tool.model_dump(exclude_none=True).items():
            # Format value based on type, ensuring XML safety
            if isinstance(field_value, bool):
                value_str = str(field_value).lower()
            elif isinstance(field_value, (list, dict)):
                # Simple string representation for list/dict for now.
                # Consider JSON within the tag if needed and supported by the prompt/LLM.
                # Use JSON for structured data
                value_str = json.dumps(field_value, ensure_ascii=False)
            else:
                value_str = str(field_value)

            # Escape the value content
            escaped_value = xml.sax.saxutils.escape(value_str)

            # Handle multi-line content like 'content' or 'diff' - ensure newlines are preserved
            if '\n' in value_str:
                # Add newline before closing tag for readability if content spans multiple lines
                xml_parts.append(
                    f"<{field_name}>\n{escaped_value}\n</{field_name}>")
            else:
                xml_parts.append(
                    f"<{field_name}>{escaped_value}</{field_name}>")

        xml_parts.append(f"</{tool_tag}>")
        # Join with newline for readability, matching prompt examples
        return "\n".join(xml_parts)
    
    @byzerllm.prompt()
    def _system(self, request: AgentRequest) -> str:
        """        
        {{system_prompt}}

        ====        

        TOOL USE

        You have access to a set of tools that are executed upon the user's approval. You can use one tool per message, and will receive the result of that tool use in the user's response. You use tools step-by-step to accomplish a given task, with each tool use informed by the result of the previous tool use.

        # Tool Use Formatting

        Tool use is formatted using XML-style tags. The tool name is enclosed in opening and closing tags, and each parameter is similarly enclosed within its own set of tags. Here's the structure:

        <tool_name>
        <parameter1_name>value1</parameter1_name>
        <parameter2_name>value2</parameter2_name>
        ...
        </tool_name>

        For example:

        <read_file>
        <path>src/main.js</path>
        </read_file>

        Always adhere to this format for the tool use to ensure proper parsing and execution.


        # Tools

        {% for tool_tag, tool_description in tool_descriptions.items() %}
        ## {{ tool_tag }}
        {{ tool_description.description }}                
        {% endfor %}        

        {%if mcp_server_info %}
        ### MCP_SERVER_LIST
        {{mcp_server_info}}
        {%endif%}

        {%if agent_info %}                   
        ### AVAILABLE_AGENTS
        {{agent_info}}
        {%endif%}

        {%if group_info %}
        ### AVAILABLE_GROUPS
        {{group_info}}
        {%endif%}

        # Tool Use Examples
        {% set example_count = 0 %}
        {% for tool_tag, example in tool_examples.items() %}
        {% if example %}
        {% set example_count = example_count + 1 %}
        ## Example {{ example_count }}: {{ example.title }}
        {{ example.body }}
        {% endif %}
        {% endfor %}                                      

        
        # Tool Use Guidelines

        1. In <thinking> tags, assess what information you already have and what information you need to proceed with the task.
        2. Choose the most appropriate tool based on the task and the tool descriptions provided. Assess if you need additional information to proceed, and which of the available tools would be most effective for gathering this information. For example using the list_files tool is more effective than running a command like `ls` in the terminal. It's critical that you think about each available tool and use the one that best fits the current step in the task.
        3. If multiple actions are needed, use one tool at a time per message to accomplish the task iteratively, with each tool use being informed by the result of the previous tool use. Do not assume the outcome of any tool use. Each step must be informed by the previous step's result.
        4. Formulate your tool use using the XML format specified for each tool.
        5. After each tool use, the user will respond with the result of that tool use. This result will provide you with the necessary information to continue your task or make further decisions. This response may include:
        - Information about whether the tool succeeded or failed, along with any reasons for failure.
        - Linter errors that may have arisen due to the changes you made, which you'll need to address.
        - New terminal output in reaction to the changes, which you may need to consider or act upon.
        - Any other relevant feedback or information related to the tool use.
        6. ALWAYS wait for user confirmation after each tool use before proceeding. Never assume the success of a tool use without explicit confirmation of the result from the user.
        {% for tool_name, guideline in tool_guidelines.items() %}                
        {{ loop.index + 6 }}. **{{ tool_name }}**: {{ guideline }}
        {% endfor %}

        It is crucial to proceed step-by-step, waiting for the user's message after each tool use before moving forward with the task. This approach allows you to:
        1. Confirm the success of each step before proceeding.
        2. Address any issues or errors that arise immediately.
        3. Adapt your approach based on new information or unexpected results.
        4. Ensure that each action builds correctly on the previous ones.        
        

        By waiting for and carefully considering the user's response after each tool use, you can react accordingly and make informed decisions about how to proceed with the task. This iterative process helps ensure the overall success and accuracy of your work.
                
        {% for case_name, case_info in tool_case_docs.items() %}
        
        ====

        # {{ case_name | upper }}
        
        {{ case_info.doc }}
        {% endfor %}
        
        {% if enable_active_context_in_generate %}
        ====

        PROJECT PACKAGE CONTEXT

        Each directory can contain a short **`active.md`** summary file located under the mirrored path inside
        `{{ current_project }}/.auto-coder/active-context/`.

        * **Purpose** – captures only the files that have **recently changed** in that directory. It is *not* a full listing.
        * **Example** – for `{{ current_project }}/src/abc/bbc`, the summary is
          `{{ current_project }}/.auto-coder/active-context/src/abc/bbc/active.md`.

        **Reading a summary**

        ```xml
        <read_file>
        <path>.auto-coder/active-context/src/abc/bbc/active.md</path>
        </read_file>
        ```

        Use these summaries to quickly decide which files deserve a deeper look with tools like
        `read_file`, `search_files`, or `list_code_definition_names`.

        {% endif %}
        ====

        CAPABILITIES

        - You are a powerful deep research RAG system, specialized in collecting and synthesizing information to answer complex user queries.
        - You can execute commands, list files, perform regex searches, read file contents, ask follow-up questions, and more to help you thoroughly research any topic.
        - When a user presents a task, a list of file paths in the current working directory ('{{ current_project }}') is included in environment_details, providing an overview of the project structure.
        - You can use the search_files tool to perform regex searches in specified directories, with results including context. This is particularly useful for understanding code patterns, finding specific implementations, or identifying areas that need attention.
        - You can use the execute_command tool to run commands on the user's computer to perform specific system operations. Commands will be executed in the current working directory.
        - Remember, the write_to_file and replace_in_file tools are ONLY for creating and updating research plans, search strategies, or summarizing findings—never for modifying system files or operational code.

        ====

        RULES

        - Your current working directory is: {{current_project}}
        - You cannot `cd` into a different directory to complete a task. You are stuck operating from '{{ current_project }}', so be sure to pass in the correct 'path' parameter when using tools that require a path.
        - Do not use the ~ character or $HOME to refer to the home directory.
        - Before using the execute_command tool, you must first think about the SYSTEM INFORMATION context provided to understand the user's environment and tailor your commands to ensure they are compatible with their system. You must also consider if the command you need to run should be executed in a specific directory outside of the current working directory '{{ current_project }}', and if so prepend with `cd`'ing into that directory && then executing the command (as one command since you are stuck operating from '{{current_project}}'). For example, if you needed to run `npm install` in a project outside of '{{current_project}}', you would need to prepend with a `cd` i.e. pseudocode for this would be `cd (path to project) && (command, in this case npm install)`.
        - When using the search_files tool, craft your regex patterns carefully to balance specificity and flexibility. Based on the user's task you may use it to find code patterns, TODO comments, function definitions, or any text-based information across the project. The results include context, so analyze the surrounding code to better understand the matches. Leverage the search_files tool in combination with other tools for more comprehensive analysis. For example, use it to find specific code patterns, then use read_file to examine the full context of interesting matches.
        - The write_to_file and replace_in_file tools are ONLY to be used for creating and updating research plans, search strategies, or summarizing findings. They are NOT to be used for modifying system files or any operational code.
        - When making research plans, always consider the context and objectives clearly outlined by the user.
        - Do not ask for more information than necessary. Use the tools provided to accomplish the user's request efficiently and effectively. When you've completed your task, you must use the attempt_completion tool to present the result to the user. The user may provide feedback, which you can use to make improvements and try again.
        {% if enable_tool_ask_followup_question %}
        - You are only allowed to ask the user questions using the ask_followup_question tool. Use this tool only when you need additional details to complete a task, and be sure to use a clear and concise question that will help you move forward with the task. However if you can use the available tools to avoid having to ask the user questions, you should do so. For example, if the user mentions a file that may be in an outside directory like the Desktop, you should use the list_files tool to list the files in the Desktop and check if the file they are talking about is there, rather than asking the user to provide the file path themselves.
        - When executing commands, if you don't see the expected output, assume the terminal executed the command successfully and proceed with the task. The user's terminal may be unable to stream the output back properly. If you absolutely need to see the actual terminal output, use the ask_followup_question tool to request the user to copy and paste it back to you.
        {% endif %}
        - The user may provide a file's contents directly in their message, in which case you shouldn't use the read_file tool to get the file contents again since you already have it.
        - Your goal is to try to accomplish the user's task, NOT engage in a back and forth conversation.
        - NEVER end attempt_completion result with a question or request to engage in further conversation! Formulate the end of your result in a way that is final and does not require further input from the user.
        - You are STRICTLY FORBIDDEN from starting your messages with "Great", "Certainly", "Okay", "Sure". You should NOT be conversational in your responses, but rather direct and to the point. For example you should NOT say "Great, I've updated the CSS" but instead something like "I've updated the research findings". It is important you be clear and technical in your messages.
        - When presented with images, utilize your vision capabilities to thoroughly examine them and extract meaningful information. Incorporate these insights into your thought process as you accomplish the user's task.
        - At the end of each user message, you will automatically receive environment_details. This information is not written by the user themselves, but is auto-generated to provide potentially relevant context about the project structure and environment. While this information can be valuable for understanding the project context, do not treat it as a direct part of the user's request or response. Use it to inform your actions and decisions, but don't assume the user is explicitly asking about or referring to this information unless they clearly do so in their message. When using environment_details, explain your actions clearly to ensure the user understands, as they may not be aware of these details.
        - Before executing commands, check the "Actively Running Terminals" section in environment_details. If present, consider how these active processes might impact your task. For example, if a local development server is already running, you wouldn't need to start it again. If no active terminals are listed, proceed with command execution as normal.
        - When using the replace_in_file tool, you must include complete lines in your SEARCH blocks, not partial lines. The system requires exact line matches and cannot match partial lines. For example, if you want to match a line containing "const x = 5;", your SEARCH block must include the entire line, not just "x = 5" or other fragments.
        - When using the replace_in_file tool, if you use multiple SEARCH/REPLACE blocks, list them in the order they appear in the file. For example if you need to make changes to both line 10 and line 50, first include the SEARCH/REPLACE block for line 10, followed by the SEARCH/REPLACE block for line 50.
        - It is critical you wait for the user's response after each tool use, in order to confirm the success of the tool use.        
        - To display LaTeX formulas, use a single dollar sign to wrap inline formulas, like `$E=mc^2$`, and double dollar signs to wrap block-level formulas, like `$$\frac{d}{dx}e^x = e^x$$`.
        - To include flowcharts or diagrams, you can use Mermaid syntax.
        
        {% if extra_docs %}  
        ====
      
        RULES OR  DOCUMENTS PROVIDED BY USER

        The following rules are provided by the user, and you must follow them strictly.
        
        {% for key, value in extra_docs.items() %}        
        ##File: {{ key }}
        {{ value }}        
        {% endfor %}          
        
        Make sure you always start your task by using the read_file tool to get the relevant RULE files listed in index.md based on the user's specific requirements.        
        {% endif %}

        ====

        SYSTEM INFORMATION

        Operating System: {{os_distribution}}
        Default Shell: {{shell_type}}
        Home Directory: {{home_dir}}
        Current Working Directory: {{current_project}}

        ====

        OBJECTIVE

        You accomplish a given task iteratively, breaking it down into clear steps and working through them methodically.

        1. Analyze the user's query and identify clear, achievable research objectives. Prioritize these objectives in a logical order.
        2. Sequentially use available tools to gather information, using only one tool at a time. Each objective should correspond to a distinct step in your problem-solving process.
        3. Before using tools, analyze within <thinking></thinking> tags. First examine the file structure in environment_details for context, consider which tool is most relevant, then analyze whether you have enough information for each required parameter. If all required parameters exist or can be reasonably inferred, close the thinking tag and proceed with the tool. If a required parameter value is missing, don't call the tool but instead use ask_followup_question to request information from the user.
        4. After completing the task, use the attempt_completion tool to present your results to the user.
        5. The user may provide feedback for improvements. Avoid pointless back-and-forth conversations; don't end responses with questions or offers of further assistance.
        
        {% if file_paths_str %}
        ====
        The following are files that the user is currently focusing on. 
        Make sure you always start your analysis by using the read_file tool to get the content of the files.
        <files>
        {{file_paths_str}}
        </files>
        {% endif %}
        """
        return self._render_context()
        
    
    def _render_context(self):                
        # 获取工具描述和示例
        tool_descriptions = ToolRegistry.get_all_tool_descriptions()
        tool_examples = ToolRegistry.get_all_tool_examples()
        tool_case_docs = ToolRegistry.get_all_tools_case_docs()
        tool_guidelines = ToolRegistry.get_all_tool_use_guidelines()
        
        extra_docs = get_required_and_index_rules()  
        
        env_info = detect_env()
        shell_type = "bash"
        if shells.is_running_in_cmd():
            shell_type = "cmd"
        elif shells.is_running_in_powershell():
            shell_type = "powershell"

        file_paths_str = "\n".join([file_source.module_name for file_source in self.files.sources])
        
        # 获取代理信息
        agent_info = ""
        agent_names = AgentHub.list_agents()
        if agent_names:
            agent_info = "Available Agents:\n"
            for name in agent_names:                
                agent = AgentHub.get_agent(name)
                if agent:
                    role = getattr(agent, "custom_system_prompt", "No description")
                    if name == self.name:
                        agent_info += f"- {name} (This is you, do not talk to yourself): {role[:100]}{'...' if len(role) > 100 else ''}\n"
                    else:
                        agent_info += f"- {name}: {role[:100]}{'...' if len(role) > 100 else ''}\n"
        
        # 获取群组信息
        group_info = ""
        groups = AgentHub.get_all_groups()
        if groups:
            group_info = "Available Groups:\n"
            for group in groups:
                members = []
                with group._members_lock:
                    members = [member.name for member in group.members]
                group_info += f"- {group.name}: {len(members)} members ({', '.join(members)})\n"
        
        return {
            "conversation_history": self.conversation_history,
            "env_info": env_info,
            "shell_type": shell_type,
            "shell_encoding": shells.get_terminal_encoding(),
            "conversation_safe_zone_tokens": self._get_parsed_safe_zone_tokens(),
            "os_distribution": shells.get_os_distribution(),
            "current_user": shells.get_current_username(),
            "current_project": os.path.abspath(self.args.source_dir),
            "home_dir": os.path.expanduser("~"),
            "files": self.files.to_str(),
            "mcp_server_info": self.mcp_server_info,
            "agent_info": agent_info,
            "group_info": group_info,
            "enable_active_context_in_generate": self.args.enable_active_context_in_generate,
            "extra_docs": extra_docs,
            "file_paths_str": file_paths_str,            
            "tool_descriptions": tool_descriptions,
            "tool_examples": tool_examples,
            "tool_case_docs": tool_case_docs,
            "tool_guidelines": tool_guidelines,
            "system_prompt": self.custom_system_prompt,
            "name": self.name
        }
    
    def agentic_run(self, request: AgentRequest) -> Generator[Union[LLMOutputEvent, LLMThinkingEvent, ToolCallEvent, ToolResultEvent, CompletionEvent, ErrorEvent], None, None]:
        """
        Analyzes the user request, interacts with the LLM, parses responses,
        executes tools, and yields structured events for visualization until completion or error.
        """
        logger.info(f"Starting analyze method with user input: {request.user_input[:50]}...")
        system_prompt = self._system.prompt(request)
        logger.info(f"Generated system prompt with length: {len(system_prompt)}")
        
        # print(system_prompt)
        self.agentic_conversations.clear()
        conversations = self.agentic_conversations
        conversations = [
            {"role": "system", "content": system_prompt},
        ] 
        
        # 恢复对话历史
        if self.conversation_history:
            logger.info(f"Restoring conversation history with {len(self.conversation_history)} messages")
            for message in self.conversation_history:
                # 确保消息格式正确（包含 role 和 content 字段）
                if isinstance(message, dict) and 'role' in message and 'content' in message:
                    conversations.append({
                        "role": message['role'],
                        "content": message['content']
                    })
                                
        conversations.append({
            "role": "user", "content": request.user_input
        })        
        
        logger.info(
            f"Initial conversation history size: {len(conversations)}")
        
        logger.info(f"Conversation history: {json.dumps(conversations, indent=2,ensure_ascii=False)}")

        iteration_count = 0
        tool_executed = False
        
        should_yield_completion_event = False   
        completion_event = None    

        # 更明确的循环条件，添加最大轮次检查
        max_iterations = getattr(self.args, 'agentic_max_rounds', 20)
        while iteration_count <= max_iterations:
            iteration_count += 1            
            tool_executed = False
            global_cancel.check_and_raise(token=self.cancel_token or self.args.cancel_token)            
                
            last_message = conversations[-1]
            if last_message["role"] == "assistant":
                logger.info(f"Last message is assistant, skipping LLM interaction cycle")
                if should_yield_completion_event:
                    if completion_event is None:
                        yield CompletionEvent(completion=AttemptCompletionTool(
                            result=last_message["content"],
                            command=""
                        ), completion_xml="")  
                    else:
                        yield completion_event 
                break            

            assistant_buffer = ""            

            # Check if we need to handle max rounds
            if hasattr(self.args, 'agentic_max_rounds') and iteration_count > self.args.agentic_max_rounds:
                logger.info(f"Agentic max rounds reached: {self.args.agentic_max_rounds}")
                yield CompletionEvent(completion=AttemptCompletionTool(
                    result="Agentic max rounds reached, please try again later",
                    command=""
                ), completion_xml="")
                break

            ## 实际请求大模型
            llm_response_gen = stream_chat_with_continue(
                llm=self.llm,
                conversations=conversations,
                llm_config={},  # Placeholder for future LLM configs
                args=self.args
            )
            
            parsed_events = self.stream_and_parse_llm_response(
                llm_response_gen)

            event_count = 0
            mark_event_should_finish = False
            for event in parsed_events:
                global_cancel.check_and_raise(token=self.cancel_token or self.args.cancel_token)
                event_count += 1

                if mark_event_should_finish:
                    if isinstance(event, TokenUsageEvent):
                        yield event
                    continue

                if isinstance(event, (LLMOutputEvent, LLMThinkingEvent)):
                    assistant_buffer += event.text                    
                    yield event  # Yield text/thinking immediately for display

                elif isinstance(event, ToolCallEvent):
                    
                    ## Check for background tasks notifications (basic implementation)
                    ## This is a simplified version compared to agentic_edit.py
                    ## Subclasses can override this for more sophisticated handling
                    
                    tool_executed = True
                    tool_obj = event.tool
                    tool_name = type(tool_obj).__name__
                    logger.info(f"Tool call detected: {tool_name}")
                    tool_xml = event.tool_xml  # Already reconstructed by parser

                    conversations.append({
                        "role": "assistant",
                        "content": assistant_buffer + tool_xml
                    })                    
                    assistant_buffer = ""  # Reset buffer after tool call

                    yield event  # Yield the ToolCallEvent for display

                    # Handle AttemptCompletion separately as it ends the loop
                    if isinstance(tool_obj, AttemptCompletionTool):                        
                        completion_event = CompletionEvent(
                            completion=tool_obj, completion_xml=tool_xml)                        
                        save_formatted_log(self.args.source_dir, json.dumps(
                            conversations, ensure_ascii=False), "agentic_conversation")
                        mark_event_should_finish = True
                        should_yield_completion_event = True
                        continue

                    if isinstance(tool_obj, PlanModeRespondTool):                                                
                        yield PlanModeRespondEvent(completion=tool_obj, completion_xml=tool_xml)
                        save_formatted_log(self.args.source_dir, json.dumps(
                            conversations, ensure_ascii=False), "agentic_conversation")
                        mark_event_should_finish = True
                        continue

                    # Resolve the tool
                    resolver_cls = ToolRegistry.get_resolver_for_tool(tool_obj)
                    if not resolver_cls:
                        logger.error(
                            f"No resolver implemented for tool {type(tool_obj).__name__}")
                        tool_result = ToolResult(
                            success=False, message="Error: Tool resolver not implemented.", content=None)
                        result_event = ToolResultEvent(tool_name=type(
                            tool_obj).__name__, result=tool_result)
                        tool_executed_xml = f"<tool_result tool_name='{type(tool_obj).__name__}' success='false'><message>Error: Tool resolver not implemented.</message><content></content></tool_result>"
                    else:
                        try:
                            logger.info(f"Creating resolver for tool: {tool_name}")
                            resolver = resolver_cls(
                                agent=self, tool=tool_obj, args=self.args)
                            logger.info(
                                f"Executing tool: {type(tool_obj).__name__} with params: {tool_obj.model_dump()}")
                            tool_result: ToolResult = resolver.resolve()
                            logger.info(
                                f"Tool Result: Success={tool_result.success}, Message='{tool_result.message}'")
                            result_event = ToolResultEvent(tool_name=type(
                                tool_obj).__name__, result=tool_result)

                            escaped_message = xml.sax.saxutils.escape(
                                tool_result.message)
                            content_str = str(
                                tool_result.content) if tool_result.content is not None else ""
                            escaped_content = xml.sax.saxutils.escape(
                                content_str)
                            tool_executed_xml = (
                                f"<tool_result tool_name='{type(tool_obj).__name__}' success='{str(tool_result.success).lower()}'>"
                                f"<message>{escaped_message}</message>"
                                f"<content>{escaped_content}</content>"
                                f"</tool_result>"
                            )
                        except Exception as e:
                            logger.exception(
                                f"Error resolving tool {type(tool_obj).__name__}: {e}")
                            error_message = f"Critical Error during tool execution: {e}"
                            tool_result = ToolResult(
                                success=False, message=error_message, content=None)
                            result_event = ToolResultEvent(tool_name=type(
                                tool_obj).__name__, result=tool_result)
                            escaped_error = xml.sax.saxutils.escape(
                                error_message)
                            tool_executed_xml = f"<tool_result tool_name='{type(tool_obj).__name__}' success='false'><message>{escaped_error}</message><content></content></tool_result>"

                    yield result_event  # Yield the ToolResultEvent for display

                    # 添加工具结果到对话历史
                    conversations.append({
                        "role": "user",  # Simulating the user providing the tool result
                        "content": tool_executed_xml
                    })

                    # 一次交互只能有一次工具，剩下的其实就没有用了，但是如果不让流式处理完，我们就无法获取服务端
                    # 返回的token消耗和计费，所以通过此标记来完成进入空转，直到流式走完，获取到最后的token消耗和计费
                    mark_event_should_finish = True                                    

                elif isinstance(event, RetryEvent):
                    logger.info(f"Retry event occurred: {event.message}")
                    yield event                    

                elif isinstance(event, ErrorEvent):
                    logger.error(f"Error event occurred: {event.message}")
                    yield event  # Pass through errors
                    # Optionally stop the process on parsing errors
                    # logger.error("Stopping analyze loop due to parsing error.")
                    # return
                elif isinstance(event, TokenUsageEvent):                    
                    yield event
            
            if not tool_executed:
                # No tool executed in this LLM response cycle
                logger.info("LLM response finished without executing a tool.")
                # Append any remaining assistant buffer to history if it wasn't followed by a tool
                if assistant_buffer:

                    last_message = conversations[-1]
                    if last_message["role"] != "assistant":
                        logger.info("Adding new assistant message")
                        conversations.append(
                            {"role": "assistant", "content": assistant_buffer})

                    elif last_message["role"] == "assistant":
                        logger.info("Appending to existing assistant message")
                        last_message["content"] += assistant_buffer

                # 添加系统提示，要求LLM必须使用工具或明确结束，而不是直接退出
                logger.info(
                    "Adding system reminder to use tools or attempt completion")

                conversations.append({
                    "role": "user",
                    "content": "NOTE: You must use an appropriate tool (such as read_file, write_to_file, execute_command, etc.) or explicitly complete the task (using attempt_completion). Do not provide text responses without taking concrete actions. Please select a suitable tool to continue based on the user's task."
                })
                                
                continue
            
        logger.info(f"AgenticEdit analyze loop finished after {iteration_count} iterations.")
        save_formatted_log(self.args.source_dir, json.dumps(conversations, ensure_ascii=False), "agentic_conversation")
    
    def stream_and_parse_llm_response(
        self, generator: Generator[Tuple[str, Any], None, None]
    ) -> Generator[Union[LLMOutputEvent, LLMThinkingEvent, ToolCallEvent, ErrorEvent], None, None]:
        """
        Streamingly parses the LLM response generator, distinguishing between
        plain text, thinking blocks, and tool usage blocks, yielding corresponding Event models.

        Args:
            generator: An iterator yielding (content, metadata) tuples from the LLM stream.

        Yields:
            Union[LLMOutputEvent, LLMThinkingEvent, ToolCallEvent, ErrorEvent]: Events representing
            different parts of the LLM's response.
        """
        buffer = ""
        in_tool_block = False
        in_thinking_block = False
        current_tool_tag = None
        
        # 预编译正则表达式以提高性能
        tool_start_pattern = re.compile(r"<([a-zA-Z0-9_]+)>")
        _tool_end_pattern = re.compile(r"</([a-zA-Z0-9_]+)>")
        thinking_start_tag = "<thinking>"
        thinking_end_tag = "</thinking>"
        _param_pattern = re.compile(r"<([a-zA-Z0-9_]+)>(.*?)</\1>", re.DOTALL)
        
        # 性能监控
        start_time = time.time()
        event_count = 0

        def parse_tool_xml(tool_xml: str, tool_tag: str) -> Optional[BaseTool]:
            """Minimal parser for tool XML string."""
            params = {}
            try:
                # Find content between <tool_tag> and </tool_tag>
                inner_xml_match = re.search(
                    rf"<{tool_tag}>(.*?)</{tool_tag}>", tool_xml, re.DOTALL)
                if not inner_xml_match:
                    logger.error(
                        f"Could not find content within <{tool_tag}>...</{tool_tag}>")
                    return None
                inner_xml = inner_xml_match.group(1).strip()

                # Find <param>value</param> pairs within the inner content
                pattern = re.compile(r"<([a-zA-Z0-9_]+)>(.*?)</\1>", re.DOTALL)
                for m in pattern.finditer(inner_xml):
                    key = m.group(1)
                    # Basic unescaping (might need more robust unescaping if complex values are used)
                    val = xml.sax.saxutils.unescape(m.group(2))
                    params[key] = val

                tool_cls = ToolRegistry.get_model_for_tag(tool_tag)
                if tool_cls:
                    # Attempt to handle boolean conversion specifically for requires_approval
                    if 'requires_approval' in params:
                        params['requires_approval'] = params['requires_approval'].lower(
                        ) == 'true'
                    # Attempt to handle JSON parsing for ask_followup_question_tool
                    if tool_tag == 'ask_followup_question' and 'options' in params:
                        try:
                            params['options'] = json.loads(
                                params['options'])
                        except json.JSONDecodeError:
                            logger.warning(
                                f"Could not decode JSON options for ask_followup_question_tool: {params['options']}")
                            # Keep as string or handle error? Let's keep as string for now.
                            pass
                    if tool_tag == 'plan_mode_respond' and 'options' in params:
                        try:
                            params['options'] = json.loads(
                                params['options'])
                        except json.JSONDecodeError:
                            logger.warning(
                                f"Could not decode JSON options for plan_mode_respond_tool: {params['options']}")
                    # Handle recursive for list_files
                    if tool_tag == 'list_files' and 'recursive' in params:
                        params['recursive'] = params['recursive'].lower() == 'true'
                    return tool_cls(**params)
                else:
                    logger.error(f"Tool class not found for tag: {tool_tag}")
                    return None
            except Exception as e:
                logger.exception(
                    f"Failed to parse tool XML for <{tool_tag}>: {e}\nXML:\n{tool_xml}")
                return None
        
        last_metadata = None      
        for content_chunk, metadata in generator:
            global_cancel.check_and_raise(token=self.cancel_token or self.args.cancel_token)                      
            if not content_chunk:
                last_metadata = metadata                
                continue
            
            last_metadata = metadata
            buffer += content_chunk  

            while True:
                # Check for transitions: thinking -> text, tool -> text, text -> thinking, text -> tool
                next_event_pos = len(buffer)
                found_event = False

                # 1. Check for </thinking> if inside thinking block
                if in_thinking_block:
                    end_think_pos = buffer.find(thinking_end_tag)
                    if end_think_pos != -1:
                        thinking_content = buffer[:end_think_pos]
                        yield LLMThinkingEvent(text=thinking_content)
                        buffer = buffer[end_think_pos + len(thinking_end_tag):]
                        in_thinking_block = False
                        found_event = True
                        continue  # Restart loop with updated buffer/state
                    else:
                        # Need more data to close thinking block
                        break

                # 2. Check for </tool_tag> if inside tool block
                elif in_tool_block:
                    end_tag = f"</{current_tool_tag}>"
                    end_tool_pos = buffer.find(end_tag)
                    if end_tool_pos != -1:
                        tool_block_end_index = end_tool_pos + len(end_tag)
                        tool_xml = buffer[:tool_block_end_index]
                        tool_obj = parse_tool_xml(tool_xml, current_tool_tag)

                        if tool_obj:
                            # Reconstruct the XML accurately here AFTER successful parsing
                            # This ensures the XML yielded matches what was parsed.
                            reconstructed_xml = self._reconstruct_tool_xml(
                                tool_obj)
                            if reconstructed_xml.startswith("<error>"):
                                yield ErrorEvent(message=f"Failed to reconstruct XML for tool {current_tool_tag}")
                            else:
                                yield ToolCallEvent(tool=tool_obj, tool_xml=reconstructed_xml)
                        else:
                            # yield ErrorEvent(message=f"Failed to parse tool: <{current_tool_tag}>")
                            # Optionally yield the raw XML as plain text?
                            yield LLMOutputEvent(text=f"Failed to parse tool: <{current_tool_tag}> {tool_xml}")

                        buffer = buffer[tool_block_end_index:]
                        in_tool_block = False
                        current_tool_tag = None
                        found_event = True
                        continue  # Restart loop
                    else:
                        # Need more data to close tool block
                        break

                # 3. Check for <thinking> or <tool_tag> if in plain text state
                else:
                    start_think_pos = buffer.find(thinking_start_tag)
                    tool_match = tool_start_pattern.search(buffer)
                    start_tool_pos = tool_match.start() if tool_match else -1
                    tool_name = tool_match.group(1) if tool_match else None

                    # Determine which tag comes first (if any)
                    first_tag_pos = -1
                    is_thinking = False
                    is_tool = False

                    if start_think_pos != -1 and (start_tool_pos == -1 or start_think_pos < start_tool_pos):
                        first_tag_pos = start_think_pos
                        is_thinking = True
                    elif start_tool_pos != -1 and (start_think_pos == -1 or start_tool_pos < start_think_pos):
                        # Check if it's a known tool
                        if tool_name in ToolRegistry.get_tag_model_map():
                            first_tag_pos = start_tool_pos
                            is_tool = True
                        else:
                            # Unknown tag, treat as text for now, let buffer grow
                            pass

                    if first_tag_pos != -1:  # Found either <thinking> or a known <tool>
                        # Yield preceding text if any
                        preceding_text = buffer[:first_tag_pos]
                        if preceding_text:
                            yield LLMOutputEvent(text=preceding_text)

                        # Transition state
                        if is_thinking:
                            buffer = buffer[first_tag_pos +
                                            len(thinking_start_tag):]
                            in_thinking_block = True
                        elif is_tool:
                            # Keep the starting tag
                            buffer = buffer[first_tag_pos:]
                            in_tool_block = True
                            current_tool_tag = tool_name

                        found_event = True
                        continue  # Restart loop

                    else:
                        # No tags found, or only unknown tags found. Need more data or end of stream.
                        # Yield text chunk but keep some buffer for potential tag start
                        # Keep last 100 chars
                        split_point = max(0, len(buffer) - 100)
                        text_to_yield = buffer[:split_point]
                        if text_to_yield:
                            yield LLMOutputEvent(text=text_to_yield)
                            buffer = buffer[split_point:]
                        break  # Need more data

                # If no event was processed in this iteration, break inner loop
                if not found_event:
                    break               

        # After generator exhausted, yield any remaining content
        if in_thinking_block:
            # Unterminated thinking block
            yield RetryEvent(message="Stream ended with unterminated <thinking> block.")
            if buffer:
                # Yield remaining as thinking
                yield LLMThinkingEvent(text=buffer)
        elif in_tool_block:
            # Unterminated tool block
            yield RetryEvent(message=f"Stream ended with unterminated <{current_tool_tag}> block.")
            if buffer:
                yield LLMOutputEvent(text=buffer)  # Yield remaining as text
        elif buffer:
            # Yield remaining plain text
            yield LLMOutputEvent(text=buffer)

        # 性能日志
        duration = time.time() - start_time
        logger.info(
            f"Event stream processing completed: {event_count} events in {duration:.3f}s "
            f"({event_count/max(duration, 0.001):.1f} events/sec)")
            
        # 这个要放在最后，防止其他关联的多个事件的信息中断
        yield TokenUsageEvent(usage=last_metadata)     
    
    def run_with_events(self, request: AgentRequest):
        """
        运行代理过程，将内部事件转换为标准事件系统格式，并通过事件管理器写入

        Args:
            request: 代理请求
        """
        event_manager = get_event_manager(self.args.event_file)
        base_url = "/agent/base"

        try:
            event_stream = self.agentic_run(request)
            for agent_event in event_stream:
                content = None
                metadata = EventMetadata(
                    action_file=self.args.file,
                    is_streaming=False,
                    stream_out_type=base_url)

                if isinstance(agent_event, LLMThinkingEvent):
                    content = EventContentCreator.create_stream_thinking(
                        content=agent_event.text)
                    metadata.is_streaming = True
                    metadata.path = f"{base_url}/thinking"
                    event_manager.write_stream(
                        content=content.to_dict(), metadata=metadata.to_dict())
                elif isinstance(agent_event, LLMOutputEvent):
                    content = EventContentCreator.create_stream_content(
                        content=agent_event.text)
                    metadata.is_streaming = True
                    metadata.path = f"{base_url}/output"
                    event_manager.write_stream(content=content.to_dict(),
                                               metadata=metadata.to_dict())
                elif isinstance(agent_event, ToolCallEvent):
                    tool_name = type(agent_event.tool).__name__
                    metadata.path = f"{base_url}/tool/call"
                    content = EventContentCreator.create_result(
                        content={
                            "tool_name": tool_name,
                            **agent_event.tool.model_dump()
                        },
                        metadata={}
                    )
                    event_manager.write_result(
                        content=content.to_dict(), metadata=metadata.to_dict())
                elif isinstance(agent_event, ToolResultEvent):
                    metadata.path = f"{base_url}/tool/result"
                    content = EventContentCreator.create_result(
                        content={
                            "tool_name": agent_event.tool_name,
                            **agent_event.result.model_dump()
                        },
                        metadata={}
                    )
                    event_manager.write_result(
                        content=content.to_dict(), metadata=metadata.to_dict())
                elif isinstance(agent_event, TokenUsageEvent):
                    # 缓存模型价格信息
                    if not hasattr(self, '_cached_model_info'):
                        model_name = ",".join(llms_utils.get_llm_names(self.llm))
                        self._cached_model_info = llms_utils.get_model_info(
                            model_name, self.args.product_mode) or {}
                        self._cached_model_name = model_name

                    # 获取价格
                    input_price = self._cached_model_info.get("input_price", 0.0)
                    output_price = self._cached_model_info.get("output_price", 0.0)

                    # 计算成本
                    last_meta = agent_event.usage
                    input_tokens = max(last_meta.input_tokens_count, 0)
                    output_tokens = max(last_meta.generated_tokens_count, 0)
                    
                    input_cost = (input_tokens * input_price) / 1000000  # 转换为百万
                    output_cost = (output_tokens * output_price) / 1000000

                    # 预测总成本
                    if not hasattr(self, '_total_input_tokens'):
                        self._total_input_tokens = 0
                        self._total_output_tokens = 0
                        self._total_input_cost = 0.0
                        self._total_output_cost = 0.0
                    
                    self._total_input_tokens += input_tokens
                    self._total_output_tokens += output_tokens
                    self._total_input_cost += input_cost
                    self._total_output_cost += output_cost

                    # 记录详细Token使用情况
                    logger.info(
                        f"Token Usage Details:\n"
                        f"- Model: {self._cached_model_name}\n"
                        f"- Input: {input_tokens} tokens (${input_cost:.6f})\n"
                        f"- Output: {output_tokens} tokens (${output_cost:.6f})\n"
                        f"- Running Totals:\n"
                        f"  - Input: {self._total_input_tokens} tokens (${self._total_input_cost:.6f})\n"
                        f"  - Output: {self._total_output_tokens} tokens (${self._total_output_cost:.6f})\n"
                        f"- Estimated Final Cost: ${(self._total_input_cost + self._total_output_cost) * 1.2:.6f} (with 20% buffer)"
                    )

                    # 写入Token统计事件
                    get_event_manager(self.args.event_file).write_result(
                        EventContentCreator.create_result(content=EventContentCreator.ResultTokenStatContent(
                            model_name=model_name,
                            elapsed_time=0.0,
                            first_token_time=last_meta.first_token_time,
                            input_tokens=last_meta.input_tokens_count,
                            output_tokens=last_meta.generated_tokens_count,
                            input_cost=input_cost,
                            output_cost=output_cost
                        ).to_dict()), metadata=metadata.to_dict())

                elif isinstance(agent_event, CompletionEvent):
                    # 在这里完成实际合并
                    try:
                        self.apply_changes()
                    except Exception as e:
                        logger.exception(f"Error merging shadow changes to project: {e}")

                    metadata.path = f"{base_url}/completion"
                    content = EventContentCreator.create_completion(
                        success_code="AGENT_COMPLETE",
                        success_message="Agent attempted task completion.",
                        result={
                            "response": agent_event.completion.result
                        }
                    )
                    event_manager.write_completion(
                        content=content.to_dict(), metadata=metadata.to_dict())
                elif isinstance(agent_event, ErrorEvent):
                    metadata.path = f"{base_url}/error"
                    content = EventContentCreator.create_error(
                        error_code="AGENT_ERROR",
                        error_message=agent_event.message,
                        details={"agent_event_type": "ErrorEvent"}
                    )
                    event_manager.write_error(
                        content=content.to_dict(), metadata=metadata.to_dict())
                else:
                    metadata.path = f"{base_url}/error"
                    logger.warning(f"未处理的代理事件类型: {type(agent_event)}")
                    content = EventContentCreator.create_error(
                        error_code="AGENT_ERROR",
                        error_message=f"未处理的代理事件类型: {type(agent_event)}",
                        details={"agent_event_type": type(agent_event).__name__}
                    )
                    event_manager.write_error(
                        content=content.to_dict(), metadata=metadata.to_dict())

        except Exception as e:
            logger.exception("代理执行过程中发生意外错误:")
            metadata = EventMetadata(
                action_file=self.args.file,
                is_streaming=False,
                stream_out_type=f"{base_url}/error")
            error_content = EventContentCreator.create_error(
                error_code="AGENT_FATAL_ERROR",
                error_message=f"发生意外错误: {str(e)}",
                details={"exception_type": type(e).__name__}
            )
            event_manager.write_error(
                content=error_content.to_dict(), metadata=metadata.to_dict())
            raise e

    def run_with_generator(self, request: AgentRequest):        
        start_time = time.time()             
        project_name = os.path.basename(os.path.abspath(self.args.source_dir))
        yield ("thinking",agentic_lang.get_message_with_format("agent_start", project_name=project_name))

        # 添加token累计变量
        total_input_tokens = 0
        total_output_tokens = 0
        total_input_cost = 0.0
        total_output_cost = 0.0
        model_name = ""

        try:
            event_stream = self.agentic_run(request)
            for event in event_stream:
                if isinstance(event, TokenUsageEvent):
                    # 获取模型信息以计算价格
                    from autocoder.utils import llms as llm_utils
                    model_name = ",".join(llm_utils.get_llm_names(self.llm))
                    model_info = llm_utils.get_model_info(
                        model_name, self.args.product_mode) or {}
                    input_price = model_info.get(
                        "input_price", 0.0) if model_info else 0.0
                    output_price = model_info.get(
                        "output_price", 0.0) if model_info else 0.0

                    # 计算成本
                    last_meta = event.usage
                    input_cost = (last_meta.input_tokens_count *
                                 input_price) / 1000000  # 转换为百万
                    output_cost = (
                        last_meta.generated_tokens_count * output_price) / 1000000

                    # 累计token使用情况
                    total_input_tokens += last_meta.input_tokens_count
                    total_output_tokens += last_meta.generated_tokens_count
                    total_input_cost += input_cost
                    total_output_cost += output_cost

                    # 记录日志
                    logger.info(agentic_lang.get_message_with_format(
                        "token_usage_log", 
                        model=model_name, 
                        input_tokens=last_meta.input_tokens_count, 
                        output_tokens=last_meta.generated_tokens_count, 
                        input_cost=input_cost, 
                        output_cost=output_cost))
                    
                elif isinstance(event, LLMThinkingEvent):
                    # 以较不显眼的样式呈现思考内容
                    yield ("thinking",f"{event.text}")
                elif isinstance(event, LLMOutputEvent):
                    # 打印常规LLM输出
                    yield ("thinking",event.text)
                elif isinstance(event, ToolCallEvent):
                    # 跳过显示完成工具的调用
                    if hasattr(event.tool, "result") and hasattr(event.tool, "command"):
                        continue  # 不显示完成工具的调用

                    tool_name = type(event.tool).__name__
                    # 使用工具展示函数（需要自行实现）
                    display_content = get_tool_display_message(event.tool)                    
                    yield ("thinking", agentic_lang.get_message_with_format("tool_operation_title", tool_name=tool_name))
                    yield ("thinking",display_content)

                elif isinstance(event, ToolResultEvent):
                    # 跳过显示完成工具的结果
                    if event.tool_name == "AttemptCompletionTool":
                        continue

                    result = event.result 
                    success_status = agentic_lang.get_message("success_status") if result.success else agentic_lang.get_message("failure_status")                   
                    base_content = agentic_lang.get_message_with_format("status", status=success_status) + "\n"
                    base_content += agentic_lang.get_message_with_format("message", message=result.message) + "\n"

                    # 格式化内容函数
                    def _format_content(content):
                        if len(content) > 200:
                            return f"{content[:100]}\n...\n{content[-100:]}"
                        else:
                            return content

                    # 首先准备基本信息面板
                    panel_content = [base_content]
                    syntax_content = None

                    if result.content is not None:
                        content_str = ""
                        try:
                            if isinstance(result.content, (dict, list)):
                                import json
                                content_str = json.dumps(
                                    result.content, indent=2, ensure_ascii=False)
                                syntax_content = _format_content(content_str)                                    
                            elif isinstance(result.content, str) and ('\n' in result.content or result.content.strip().startswith('<')):                                                                
                                syntax_content = _format_content(result.content)
                            else:
                                content_str = str(result.content)
                                # 直接附加简单字符串内容
                                panel_content.append(
                                    _format_content(content_str))
                        except Exception as e:
                            logger.warning(agentic_lang.get_message_with_format("format_tool_error", error=str(e)))
                            panel_content.append(
                                # 备用
                                _format_content(str(result.content)))

                    # 打印基本信息面板
                    yield ("thinking",_format_content("\n".join(panel_content)))
                    # 单独打印语法高亮内容（如果存在）
                    if syntax_content:
                        yield ("thinking",syntax_content)

                elif isinstance(event, CompletionEvent):                    
                    yield ("result",event.completion.result)
                    if event.completion.command:
                        yield ("result", agentic_lang.get_message_with_format("suggested_command", command=event.completion.command))
                elif isinstance(event, RetryEvent):
                    yield ("thinking", agentic_lang.get_message_with_format("retry_message", message=event.message))
                elif isinstance(event, ErrorEvent):
                    yield ("result", agentic_lang.get_message("error_title"))
                    yield ("result", agentic_lang.get_message_with_format("error_content", message=event.message))                    

                time.sleep(0.1)  # 小延迟以获得更好的视觉流

        except Exception as e:
            logger.exception(agentic_lang.get_message("unexpected_error"))
            yield ("result",agentic_lang.get_message("fatal_error_title"))
            yield ("result",agentic_lang.get_message_with_format("fatal_error_content", error=str(e)))
            raise e
        finally:
            # 在结束时打印累计的token使用情况
            duration = time.time() - start_time            
            # yield ("result","\n" + self.printer.get_message_from_key_with_format("code_generation_complete",
            #     duration=duration,
            #     input_tokens=total_input_tokens,
            #     output_tokens=total_output_tokens,
            #     input_cost=total_input_cost,
            #     output_cost=total_output_cost,
            #     speed=0.0,
            #     model_names=model_name,
            #     sampling_count=1))            
    
    def run_in_terminal(self, request: AgentRequest):
        """
        在终端中运行代理过程，使用Rich库流式显示交互

        Args:
            request: 代理请求
        """
        start_time = time.time()
        console = Console()        
        project_name = os.path.basename(os.path.abspath(self.args.source_dir))
        console.rule(agentic_lang.get_message_with_format("agent_start", project_name=project_name))
        console.print(Panel(
            agentic_lang.get_message_with_format("user_input", input=request.user_input), 
            title=agentic_lang.get_message("user_input_title"), 
            border_style="blue"))

        # 添加token累计变量
        total_input_tokens = 0
        total_output_tokens = 0
        total_input_cost = 0.0
        total_output_cost = 0.0
        model_name = ""

        try:
            event_stream = self.agentic_run(request)
            for event in event_stream:
                if isinstance(event, TokenUsageEvent):
                    # 获取模型信息以计算价格
                    from autocoder.utils import llms as llm_utils
                    model_name = ",".join(llm_utils.get_llm_names(self.llm))
                    model_info = llm_utils.get_model_info(
                        model_name, self.args.product_mode) or {}
                    input_price = model_info.get(
                        "input_price", 0.0) if model_info else 0.0
                    output_price = model_info.get(
                        "output_price", 0.0) if model_info else 0.0

                    # 计算成本
                    last_meta = event.usage
                    input_cost = (last_meta.input_tokens_count *
                                 input_price) / 1000000  # 转换为百万
                    output_cost = (
                        last_meta.generated_tokens_count * output_price) / 1000000

                    # 累计token使用情况
                    total_input_tokens += last_meta.input_tokens_count
                    total_output_tokens += last_meta.generated_tokens_count
                    total_input_cost += input_cost
                    total_output_cost += output_cost

                    # 记录日志
                    logger.info(agentic_lang.get_message_with_format(
                        "token_usage_log", 
                        model=model_name, 
                        input_tokens=last_meta.input_tokens_count, 
                        output_tokens=last_meta.generated_tokens_count, 
                        input_cost=input_cost, 
                        output_cost=output_cost))
                    
                elif isinstance(event, LLMThinkingEvent):
                    # 以较不显眼的样式呈现思考内容
                    console.print(f"[grey50]{event.text}[/grey50]", end="")
                elif isinstance(event, LLMOutputEvent):
                    # 打印常规LLM输出
                    console.print(event.text, end="")
                elif isinstance(event, ToolCallEvent):
                    # 跳过显示完成工具的调用
                    if hasattr(event.tool, "result") and hasattr(event.tool, "command"):
                        continue  # 不显示完成工具的调用

                    tool_name = type(event.tool).__name__
                    # 使用工具展示函数（需要自行实现）
                    display_content = get_tool_display_message(event.tool)
                    console.print(Panel(
                        display_content, 
                        title=agentic_lang.get_message_with_format("tool_operation_title", tool_name=tool_name), 
                        border_style="blue", 
                        title_align="left"))

                elif isinstance(event, ToolResultEvent):
                    # 跳过显示完成工具的结果
                    if event.tool_name == "AttemptCompletionTool":
                        continue

                    result = event.result
                    title = agentic_lang.get_message_with_format("tool_result_success_title", tool_name=event.tool_name) if result.success else agentic_lang.get_message_with_format("tool_result_failure_title", tool_name=event.tool_name)
                    border_style = "green" if result.success else "red"
                    success_status = agentic_lang.get_message("success_status") if result.success else agentic_lang.get_message("failure_status")
                    base_content = agentic_lang.get_message_with_format("status", status=success_status) + "\n"
                    base_content += agentic_lang.get_message_with_format("message", message=result.message) + "\n"

                    # 格式化内容函数
                    def _format_content(content):
                        return content[:200]

                    # 首先准备基本信息面板
                    panel_content = [base_content]
                    syntax_content = None

                    if result.content is not None:
                        content_str = ""
                        try:
                            if isinstance(result.content, (dict, list)):
                                import json
                                content_str = json.dumps(
                                    result.content, indent=2, ensure_ascii=False)
                                from rich.syntax import Syntax
                                syntax_content = Syntax(
                                    content_str, "json", theme="default", line_numbers=False)
                            elif isinstance(result.content, str) and ('\n' in result.content or result.content.strip().startswith('<')):
                                # 代码或XML/HTML的启发式判断
                                from rich.syntax import Syntax
                                lexer = "python"  # 默认猜测
                                if event.tool_name == "ReadFileTool" and isinstance(event.result.message, str):
                                    # 尝试从消息中的文件扩展名猜测lexer
                                    if ".py" in event.result.message:
                                        lexer = "python"
                                    elif ".js" in event.result.message:
                                        lexer = "javascript"
                                    elif ".ts" in event.result.message:
                                        lexer = "typescript"
                                    elif ".html" in event.result.message:
                                        lexer = "html"
                                    elif ".css" in event.result.message:
                                        lexer = "css"
                                    elif ".json" in event.result.message:
                                        lexer = "json"
                                    elif ".xml" in event.result.message:
                                        lexer = "xml"
                                    elif ".md" in event.result.message:
                                        lexer = "markdown"
                                    else:
                                        lexer = "text"  # 备用lexer
                                elif event.tool_name == "ExecuteCommandTool":
                                    lexer = "shell"
                                else:
                                    lexer = "text"

                                syntax_content = Syntax(
                                    _format_content(result.content), lexer, theme="default", line_numbers=True)
                            else:
                                content_str = str(result.content)
                                # 直接附加简单字符串内容
                                panel_content.append(
                                    _format_content(content_str))
                        except Exception as e:
                            logger.warning(agentic_lang.get_message_with_format("format_tool_error", error=str(e)))
                            panel_content.append(
                                # 备用
                                _format_content(str(result.content)))

                    # 打印基本信息面板
                    console.print(Panel("\n".join(
                        panel_content), title=title, border_style=border_style, title_align="left"))
                    # 单独打印语法高亮内容（如果存在）
                    if syntax_content:
                        console.print(syntax_content)

                elif isinstance(event, CompletionEvent):
                    # 在这里完成实际合并
                    try:
                        self.apply_changes()
                    except Exception as e:
                        logger.exception(agentic_lang.get_message_with_format("shadow_merge_error", error=str(e)))

                    from rich.markdown import Markdown
                    console.print(Panel(Markdown(event.completion.result),
                                  title=agentic_lang.get_message("completion_title"), 
                                  border_style="green", title_align="left"))
                    if event.completion.command:
                        console.print(agentic_lang.get_message_with_format("suggested_command", command=event.completion.command))
                elif isinstance(event, ErrorEvent):
                    console.print(Panel(
                        agentic_lang.get_message_with_format("error_content", message=event.message), 
                        title=agentic_lang.get_message("error_title"), 
                        border_style="red", title_align="left"))

                time.sleep(0.1)  # 小延迟以获得更好的视觉流

        except Exception as e:
            logger.exception(agentic_lang.get_message("unexpected_error"))
            console.print(Panel(
                agentic_lang.get_message_with_format("fatal_error_content", error=str(e)), 
                title=agentic_lang.get_message("fatal_error_title"), 
                border_style="red"))
            raise e
        finally:
            # 在结束时打印累计的token使用情况
            duration = time.time() - start_time            
            self.printer.print_in_terminal(
                "code_generation_complete",
                duration=duration,
                input_tokens=total_input_tokens,
                output_tokens=total_output_tokens,
                input_cost=total_input_cost,
                output_cost=total_output_cost,
                speed=0.0,
                model_names=model_name,
                sampling_count=1
            )
            console.rule(agentic_lang.get_message("agent_execution_complete"))
    
    def apply_pre_changes(self):
        # get the file name
        file_name = os.path.basename(self.args.file)
        if not self.args.skip_commit:
            try:
                get_event_manager(self.args.event_file).write_result(
                    EventContentCreator.create_result(
                        content=self.printer.get_message_from_key("/agent/edit/apply_pre_changes")), metadata=EventMetadata(
                        action_file=self.args.file,
                        is_streaming=False,
                        path="/agent/edit/apply_pre_changes",
                        stream_out_type="/agent/edit").to_dict())
                git_utils.commit_changes(
                    self.args.source_dir, f"auto_coder_pre_{file_name}")
            except Exception as e:
                self.printer.print_in_terminal("git_init_required",
                                               source_dir=self.args.source_dir, error=str(e))
                return

    def apply_changes(self):
        """
        Apply all tracked file changes to the original project directory.
        """
        for (file_path, change) in self.get_all_file_changes().items():
            # Ensure the directory exists before writing the file
            dir_path = os.path.dirname(file_path)
            if dir_path: # Ensure dir_path is not empty (for files in root)
                 os.makedirs(dir_path, exist_ok=True)

            with open(file_path, 'w', encoding='utf-8') as f:
                f.write(change.content)

        if len(self.get_all_file_changes()) > 0:
            if not self.args.skip_commit:
                try:
                    file_name = os.path.basename(self.args.file)
                    commit_result = git_utils.commit_changes(
                        self.args.source_dir,
                        f"{self.args.query}\nauto_coder_{file_name}",
                    )

                    get_event_manager(self.args.event_file).write_result(
                        EventContentCreator.create_result(
                            content=self.printer.get_message_from_key("/agent/edit/apply_changes")), metadata=EventMetadata(
                            action_file=self.args.file,
                            is_streaming=False,
                            stream_out_type="/agent/edit").to_dict())
                    action_yml_file_manager = ActionYmlFileManager(
                        self.args.source_dir)
                    action_file_name = os.path.basename(self.args.file)
                    add_updated_urls = []
                    commit_result.changed_files
                    for file in commit_result.changed_files:
                        add_updated_urls.append(
                            os.path.join(self.args.source_dir, file))

                    self.args.add_updated_urls = add_updated_urls
                    update_yaml_success = action_yml_file_manager.update_yaml_field(
                        action_file_name, "add_updated_urls", add_updated_urls)
                    if not update_yaml_success:
                        self.printer.print_in_terminal(
                            "yaml_save_error", style="red", yaml_file=action_file_name)

                    if self.args.enable_active_context:
                        active_context_manager = ActiveContextManager(
                            self.llm, self.args.source_dir)
                        task_id = active_context_manager.process_changes(
                            self.args)
                        self.printer.print_in_terminal("active_context_background_task",
                                                       style="blue",
                                                       task_id=task_id)
                    git_utils.print_commit_info(commit_result=commit_result)
                except Exception as e:
                    self.printer.print_str_in_terminal(
                        self.git_require_msg(
                            source_dir=self.args.source_dir, error=str(e)),
                        style="red"
                    )
        else:
            self.printer.print_in_terminal("no_changes_made")
    
    @byzerllm.prompt(render="jinja2")
    def git_require_msg(self, source_dir: str, error: str) -> str:
        '''
        auto_merge only works for git repositories.
         
        Try to use git init in the source directory. 
        
        ```shell
        cd {{ source_dir }}
        git init .
        ```

        Then try to run auto-coder again.
        Error: {{ error }}
        '''



"""
Interactive process wrapper using pexpect for shell command execution.

This module provides pexpect-based interactive process functionality including:
- Real-time input/output streaming with pexpect
- Cross-platform support (Unix-like systems)
- Signal handling (Ctrl-C, etc.)
- Thread-safe operations
- Pattern matching and expect functionality
"""

import os
import platform
import threading
import time
import queue
import signal
from typing import Optional, Dict, Any, Generator, Union, List, Tuple
from loguru import logger

from .exceptions import CommandExecutionError, ProcessCleanupError
from .process_cleanup import cleanup_process_tree

# Platform check
PLATFORM = platform.system()
PEXPECT_AVAILABLE = False

try:
    import pexpect
    if PLATFORM != "Windows":
        PEXPECT_AVAILABLE = True
    else:
        # Try to import pexpect for Windows (pexpect-windows)
        try:
            import pexpect.popen_spawn
            PEXPECT_AVAILABLE = True
        except ImportError:
            logger.debug("pexpect not fully available on Windows")
except ImportError:
    logger.debug("pexpect not available")


class InteractivePexpectProcess:
    """
    Interactive process wrapper using pexpect for real-time command execution.
    
    This class provides a pexpect-based interface for executing commands that
    require interactive input/output, with built-in pattern matching and
    expect functionality.
    """
    
    def __init__(
        self,
        command: Union[str, List[str]],
        cwd: Optional[str] = None,
        env: Optional[Dict[str, str]] = None,
        shell: bool = True,
        encoding: str = 'utf-8',
        timeout: Optional[float] = None,
        maxread: int = 2000,
        searchwindowsize: Optional[int] = None,
        **kwargs
    ):
        """
        Initialize interactive pexpect process.
        
        Args:
            command: Command to execute
            cwd: Working directory
            env: Environment variables
            shell: Whether to use shell
            encoding: Text encoding
            timeout: Default timeout for expect operations
            maxread: Maximum bytes to read at once
            searchwindowsize: Size of search window for pattern matching
            **kwargs: Additional pexpect arguments
        """
        self.command = command
        self.cwd = cwd
        self.env = env
        self.encoding = encoding
        self.shell = shell
        self.timeout = timeout
        self.maxread = maxread
        self.searchwindowsize = searchwindowsize
        self.pexpect_kwargs = kwargs
        
        # Check availability
        if not PEXPECT_AVAILABLE:
            raise CommandExecutionError("pexpect not available on this platform")
        
        # State management
        self.child: Optional[pexpect.spawn] = None
        self.logfile_read = None
        self.logfile_send = None
        
        # Threading
        self.output_queue: queue.Queue = queue.Queue()
        self.error_queue: queue.Queue = queue.Queue()
        self.io_thread: Optional[threading.Thread] = None
        self.alive_event = threading.Event()
        self.started_event = threading.Event()
        
        # Monitoring
        self.start_time: Optional[float] = None
        self.end_time: Optional[float] = None
        self.bytes_written = 0
        self.bytes_read = 0
        
        # Buffer for continuous reading
        self._output_buffer = ""
        self._read_lock = threading.Lock()
        
        logger.debug(f"InteractivePexpectProcess initialized: platform={PLATFORM}")
    
    def start(self) -> None:
        """Start the interactive pexpect process."""
        if self.is_alive():
            raise CommandExecutionError("Process is already running")
        
        self.start_time = time.time()
        self.alive_event.set()
        
        try:
            # Prepare command
            if isinstance(self.command, list):
                if self.shell:
                    # Join command parts for shell execution
                    cmd = ' '.join(self.command)
                else:
                    # Use first element as command, rest as args
                    cmd = self.command[0]
                    args = self.command[1:] if len(self.command) > 1 else []
            else:
                cmd = self.command
                args = []
            
            # Create pexpect spawn
            if PLATFORM == "Windows":
                # Use popen_spawn for Windows
                self.child = pexpect.popen_spawn.PopenSpawn(
                    cmd,
                    timeout=self.timeout,
                    maxread=self.maxread,
                    searchwindowsize=self.searchwindowsize,
                    encoding=self.encoding,
                    cwd=self.cwd,
                    env=self.env,
                    **self.pexpect_kwargs
                )
            else:
                # Use regular spawn for Unix-like systems
                if self.shell:
                    # Use shell to execute command
                    shell_cmd = os.environ.get("SHELL", "/bin/sh")
                    self.child = pexpect.spawn(
                        shell_cmd,
                        args=["-c", cmd],
                        timeout=self.timeout,
                        maxread=self.maxread,
                        searchwindowsize=self.searchwindowsize,
                        encoding=self.encoding,
                        cwd=self.cwd,
                        env=self.env,
                        **self.pexpect_kwargs
                    )
                else:
                    # Direct command execution
                    self.child = pexpect.spawn(
                        cmd,
                        args=args,
                        timeout=self.timeout,
                        maxread=self.maxread,
                        searchwindowsize=self.searchwindowsize,
                        encoding=self.encoding,
                        cwd=self.cwd,
                        env=self.env,
                        **self.pexpect_kwargs
                    )
            
            # Configure logging if needed
            if hasattr(self, 'logfile_read') and self.logfile_read:
                self.child.logfile_read = self.logfile_read
            if hasattr(self, 'logfile_send') and self.logfile_send:
                self.child.logfile_send = self.logfile_send
            
            # Start I/O thread for continuous reading
            self.io_thread = threading.Thread(target=self._io_worker, daemon=True)
            self.io_thread.start()
            
            # Give process a moment to start
            time.sleep(0.1)
            
            # Check if process is alive
            if not self.is_alive():
                self._cleanup()
                raise CommandExecutionError("Pexpect process failed to start")
            
            self.started_event.set()
            logger.info(f"Interactive pexpect process started: PID {self.pid}")
            
        except Exception as e:
            self.alive_event.clear()
            self._cleanup()
            raise CommandExecutionError(f"Failed to start pexpect process: {e}")
    
    def _io_worker(self) -> None:
        """I/O worker thread for continuous reading."""
        try:
            while self.alive_event.is_set() and self.child and self.child.isalive():
                try:
                    # Read data with timeout
                    data = self.child.read_nonblocking(size=1024, timeout=0.1)
                    if data:
                        with self._read_lock:
                            self._output_buffer += data
                            self.bytes_read += len(data.encode(self.encoding))
                            self.output_queue.put(data)
                    
                except pexpect.TIMEOUT:
                    # No data available, continue
                    continue
                except pexpect.EOF:
                    # Process terminated
                    self.alive_event.clear()
                    break
                except Exception as e:
                    logger.debug(f"I/O worker error: {e}")
                    break
            
        except Exception as e:
            logger.error(f"I/O worker fatal error: {e}")
        finally:
            self.alive_event.clear()
    
    def write(self, data: str) -> None:
        """
        Write data to process stdin.
        
        Args:
            data: Data to write
            
        Raises:
            CommandExecutionError: If process is not running or write fails
        """
        if not self.is_alive():
            raise CommandExecutionError("Process is not running")
        
        if not self.started_event.is_set():
            raise CommandExecutionError("Process has not started yet")
        
        try:
            if self.child:
                self.child.send(data)
                self.bytes_written += len(data.encode(self.encoding))
            else:
                raise CommandExecutionError("No pexpect child process available")
                
        except Exception as e:
            raise CommandExecutionError(f"Failed to write to process: {e}")
    
    def sendline(self, line: str = "") -> None:
        """
        Send a line to the process (adds newline).
        
        Args:
            line: Line to send
        """
        if self.child:
            self.child.sendline(line)
            self.bytes_written += len(line.encode(self.encoding)) + 1  # +1 for newline
    
    def sendcontrol(self, char: str) -> None:
        """
        Send a control character to the process.
        
        Args:
            char: Control character (e.g., 'c' for Ctrl-C)
        """
        if self.child:
            self.child.sendcontrol(char)
            self.bytes_written += 1
    
    def read_output(self, timeout: Optional[float] = None) -> Optional[str]:
        """
        Read output from process.
        
        Args:
            timeout: Timeout in seconds (None for non-blocking)
            
        Returns:
            Output string or None if no data available
        """
        try:
            if timeout is None:
                return self.output_queue.get_nowait()
            else:
                return self.output_queue.get(timeout=timeout)
        except queue.Empty:
            return None
    
    def read_error(self, timeout: Optional[float] = None) -> Optional[str]:
        """
        Read error output from process.
        
        Args:
            timeout: Timeout in seconds (None for non-blocking)
            
        Returns:
            Error string or None if no data available
        """
        try:
            if timeout is None:
                return self.error_queue.get_nowait()
            else:
                return self.error_queue.get(timeout=timeout)
        except queue.Empty:
            return None
    
    def expect(self, pattern: Union[str, List[str]], timeout: Optional[float] = None) -> int:
        """
        Wait for a pattern to appear in the output.
        
        Args:
            pattern: Pattern(s) to match
            timeout: Timeout in seconds
            
        Returns:
            Index of matched pattern (0 if single pattern)
            
        Raises:
            CommandExecutionError: If expect fails
        """
        if not self.child:
            raise CommandExecutionError("No pexpect child process available")
        
        try:
            if timeout is None:
                timeout = self.timeout
            
            result = self.child.expect(pattern, timeout=timeout)
            
            # Add matched text to output queue
            if self.child.after:
                self.output_queue.put(self.child.after)
            
            return result
            
        except pexpect.TIMEOUT as e:
            raise CommandExecutionError(f"Expect timeout: {e}")
        except pexpect.EOF as e:
            raise CommandExecutionError(f"Expect EOF: {e}")
        except Exception as e:
            raise CommandExecutionError(f"Expect error: {e}")
    
    def expect_exact(self, pattern: Union[str, List[str]], timeout: Optional[float] = None) -> int:
        """
        Wait for exact string(s) to appear in the output.
        
        Args:
            pattern: Exact string(s) to match
            timeout: Timeout in seconds
            
        Returns:
            Index of matched pattern (0 if single pattern)
        """
        if not self.child:
            raise CommandExecutionError("No pexpect child process available")
        
        try:
            if timeout is None:
                timeout = self.timeout
            
            result = self.child.expect_exact(pattern, timeout=timeout)
            
            # Add matched text to output queue
            if self.child.after:
                self.output_queue.put(self.child.after)
            
            return result
            
        except pexpect.TIMEOUT as e:
            raise CommandExecutionError(f"Expect exact timeout: {e}")
        except pexpect.EOF as e:
            raise CommandExecutionError(f"Expect exact EOF: {e}")
        except Exception as e:
            raise CommandExecutionError(f"Expect exact error: {e}")
    
    def read_lines(self, timeout: Optional[float] = 1.0) -> Generator[str, None, None]:
        """
        Generator that yields output lines.
        
        Args:
            timeout: Timeout for each read operation
            
        Yields:
            Output lines
        """
        buffer = ""
        
        while self.is_alive():
            try:
                data = self.read_output(timeout=timeout)
                if data:
                    buffer += data
                    
                    # Yield complete lines
                    while '\n' in buffer:
                        line, buffer = buffer.split('\n', 1)
                        yield line + '\n'
                else:
                    # No data available, check if process is still alive
                    if not self.is_alive():
                        break
                        
            except Exception as e:
                logger.debug(f"Error reading lines: {e}")
                break
        
        # Yield any remaining buffer content
        if buffer:
            yield buffer
    
    def interact(self, escape_character: str = chr(29), input_filter=None, output_filter=None) -> None:
        """
        Give control of the child process to the user.
        
        Args:
            escape_character: Character to exit interaction
            input_filter: Filter for input
            output_filter: Filter for output
        """
        if not self.child:
            raise CommandExecutionError("No pexpect child process available")
        
        try:
            self.child.interact(
                escape_character=escape_character,
                input_filter=input_filter,
                output_filter=output_filter
            )
        except Exception as e:
            raise CommandExecutionError(f"Interaction error: {e}")
    
    def send_signal(self, sig: int) -> None:
        """
        Send signal to process.
        
        Args:
            sig: Signal number
        """
        if not self.is_alive():
            return
        
        try:
            if self.child:
                if PLATFORM == "Windows":
                    # Windows signal handling
                    if sig == signal.SIGINT:
                        self.child.sendcontrol('c')
                    else:
                        self.child.terminate()
                else:
                    # Unix signal handling
                    self.child.kill(sig)
                    
        except Exception as e:
            logger.debug(f"Error sending signal {sig}: {e}")
    
    def terminate(self, grace_timeout: float = 5.0) -> bool:
        """
        Terminate the process gracefully.
        
        Args:
            grace_timeout: Time to wait for graceful termination
            
        Returns:
            True if terminated successfully
        """
        if not self.is_alive():
            return True
        
        logger.debug(f"Terminating pexpect process PID {self.pid}")
        
        try:
            # Send SIGTERM (or equivalent)
            if self.child:
                self.child.terminate()
            
            # Wait for graceful termination
            start_time = time.time()
            while time.time() - start_time < grace_timeout:
                if not self.is_alive():
                    break
                time.sleep(0.1)
            
            # Force kill if still alive
            if self.is_alive() and self.child:
                logger.debug(f"Force killing pexpect process PID {self.pid}")
                self.child.kill(signal.SIGKILL)
            
            # Wait for I/O thread to finish
            self.alive_event.clear()
            if self.io_thread and self.io_thread.is_alive():
                self.io_thread.join(timeout=2.0)
            
            self._cleanup()
            self.end_time = time.time()
            
            return not self.is_alive()
            
        except Exception as e:
            logger.error(f"Error terminating pexpect process: {e}")
            return False
    
    def _cleanup(self) -> None:
        """Clean up resources."""
        try:
            if self.child:
                try:
                    if self.child.isalive():
                        self.child.terminate(force=True)
                except:
                    pass
                
                # Close file descriptors
                try:
                    self.child.close()
                except:
                    pass
                
                self.child = None
                    
        except Exception as e:
            logger.debug(f"Cleanup error: {e}")
    
    def is_alive(self) -> bool:
        """Check if process is alive."""
        if self.child:
            return self.child.isalive()
        return False
    
    @property
    def pid(self) -> Optional[int]:
        """Get process PID."""
        if self.child:
            return self.child.pid
        return None
    
    @property
    def exit_code(self) -> Optional[int]:
        """Get process exit code."""
        if self.child:
            return self.child.exitstatus
        return None
    
    @property
    def signal_status(self) -> Optional[int]:
        """Get process signal status."""
        if self.child:
            return self.child.signalstatus
        return None
    
    @property
    def duration(self) -> Optional[float]:
        """Get process duration."""
        if self.start_time is None:
            return None
        
        end_time = self.end_time or time.time()
        return end_time - self.start_time
    
    @property
    def before(self) -> Optional[str]:
        """Get text before the last match."""
        if self.child:
            return self.child.before
        return None
    
    @property
    def after(self) -> Optional[str]:
        """Get text after the last match."""
        if self.child:
            return self.child.after
        return None
    
    @property
    def match(self) -> Optional[str]:
        """Get the last match object."""
        if self.child:
            return self.child.match
        return None
    
    def get_stats(self) -> Dict[str, Any]:
        """Get process statistics."""
        return {
            'pid': self.pid,
            'exit_code': self.exit_code,
            'signal_status': self.signal_status,
            'duration': self.duration,
            'bytes_written': self.bytes_written,
            'bytes_read': self.bytes_read,
            'is_alive': self.is_alive(),
            'platform': PLATFORM,
            'pexpect_available': PEXPECT_AVAILABLE
        }
    
    def __enter__(self) -> 'InteractivePexpectProcess':
        """Context manager entry."""
        self.start()
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        """Context manager exit."""
        self.terminate()
    
    def __del__(self) -> None:
        """Destructor - cleanup resources."""
        try:
            if self.is_alive():
                self.terminate(grace_timeout=1.0)
        except Exception:
            pass 


"""
Todo manager exception classes.
"""


class TodoManagerError(Exception):
    """Base exception for todo manager operations"""
    
    def __init__(self, message="Todo manager error occurred", error_code="GENERAL_ERROR"):
        super().__init__(message)
        self.error_code = error_code


class TodoNotFoundError(TodoManagerError):
    """Exception raised when a todo is not found"""
    
    def __init__(self, todo_id):
        if (isinstance(todo_id, str) and len(todo_id) > 0 and 
            not any(ord(c) > 127 or c.isspace() for c in todo_id)):
            message = f"Todo not found: {todo_id}"
        else:
            message = todo_id  # Custom message
        super().__init__(message, error_code="TODO_NOT_FOUND")


class TodoListNotFoundError(TodoManagerError):
    """Exception raised when a todo list is not found"""
    
    def __init__(self, conversation_id):
        if (isinstance(conversation_id, str) and len(conversation_id) > 0 and 
            not any(ord(c) > 127 or c.isspace() for c in conversation_id)):
            message = f"Todo list not found for conversation: {conversation_id}"
        else:
            message = conversation_id  # Custom message
        super().__init__(message, error_code="TODO_LIST_NOT_FOUND") 


from autocoder.common import SourceCode, AutoCoderArgs
from autocoder import common as FileUtils
from autocoder.utils.rest import HttpDoc
import os
from typing import Optional, Generator, List, Dict, Any, Callable
from git import Repo
import byzerllm
from autocoder.common.search import Search, SearchEngine
from loguru import logger
import re
from pydantic import BaseModel, Field
from rich.console import Console
import json
from autocoder.common import files as FileUtils
from autocoder.common.ignorefiles import should_ignore


class DefaultProject:
    """
    DefaultProject类用于遍历项目目录并收集文件，使用ignorefiles模块进行文件过滤。
    与SuffixProject不同，它不按文件后缀过滤，而是收集所有不被忽略的文件。
    """

    def __init__(
        self,
        args: AutoCoderArgs,
        llm: Optional[byzerllm.ByzerLLM] = None,
        file_filter=None,
    ):
        self.args = args
        self.directory = args.source_dir
        self.git_url = args.git_url
        self.target_file = args.target_file
        self.file_filter = file_filter
        self.sources = []
        self.llm = llm
        # DefaultProject不需要exclude_files和exclude_patterns，因为使用ignorefiles模块处理

    def output(self):
        with open(self.target_file, "r", encoding="utf-8") as file:
            return file.read()

    def is_text_file(self, file_path):
        """判断是否为文本文件"""
        try:
            # 检查常见的文本文件后缀
            text_extensions = {
                ".txt",
                ".md",
                ".py",
                ".js",
                ".ts",
                ".jsx",
                ".tsx",
                ".html",
                ".htm",
                ".css",
                ".scss",
                ".sass",
                ".less",
                ".json",
                ".xml",
                ".yaml",
                ".yml",
                ".toml",
                ".ini",
                ".cfg",
                ".conf",
                ".log",
                ".sql",
                ".sh",
                ".bash",
                ".zsh",
                ".fish",
                ".ps1",
                ".bat",
                ".cmd",
                ".dockerfile",
                ".gitignore",
                ".gitattributes",
                ".editorconfig",
                ".eslintrc",
                ".prettierrc",
                ".babelrc",
                ".env",
                ".properties",
                ".lock",
                ".sum",
                ".mod",
                ".go",
                ".rs",
                ".php",
                ".rb",
                ".pl",
                ".r",
                ".swift",
                ".kt",
                ".java",
                ".c",
                ".cpp",
                ".cc",
                ".cxx",
                ".h",
                ".hpp",
                ".cs",
                ".vb",
                ".fs",
                ".clj",
                ".scala",
                ".groovy",
                ".dart",
                ".elm",
                ".haskell",
                ".hs",
                ".ml",
                ".ocaml",
                ".ex",
                ".exs",
                ".erl",
                ".lua",
                ".vim",
                ".emacs",
                ".lisp",
                ".scheme",
                ".racket",
                ".clojure",
                ".f90",
                ".f95",
                ".f03",
                ".f08",
                ".for",
                ".ftn",
                ".m",
                ".mm",
                ".s",
                ".asm",
                ".nasm",
                ".masm",
                ".pas",
                ".pp",
                ".inc",
                ".dpr",
                ".dfm",
                ".lfm",
                ".lpr",
                ".lpk",
                ".tex",
                ".bib",
                ".sty",
                ".cls",
                ".dtx",
                ".ins",
                ".aux",
                ".toc",
                ".lof",
                ".lot",
                ".idx",
                ".ind",
                ".gls",
                ".glo",
                ".acn",
                ".acr",
                ".alg",
                ".bbl",
                ".blg",
                ".fdb_latexmk",
                ".fls",
                ".figlist",
                ".makefile",
                ".cmake",
                ".mk",
                ".am",
                ".in",
                ".ac",
                ".m4",
                ".autotools",
                ".spec",
                ".ebuild",
                ".pkgbuild",
                ".install",
                ".desktop",
                ".service",
                ".socket",
                ".timer",
                ".mount",
                ".automount",
                ".swap",
                ".target",
                ".path",
                ".slice",
                ".scope",
                ".busname",
                ".device",
                ".network",
                ".netdev",
                ".link",
                ".nspawn",
                ".container",
                ".zone",
                ".policy",
                ".rules",
                ".preset",
                ".wants",
                ".requires",
                ".requisite",
                ".conflicts",
                ".before",
                ".after",
                ".onboot",
                ".onfailure",
                ".onabort",
                ".onerror",
                ".onsuccess",
                ".onfinal",
                ".onrestart",
                ".onstop",
                ".onstart",
                ".onreload",
                ".template",
                ".d",
                ".conf.d",
                ".service.d",
                ".socket.d",
                ".timer.d",
                ".mount.d",
                ".automount.d",
                ".swap.d",
                ".target.d",
                ".path.d",
                ".slice.d",
                ".scope.d",
                ".busname.d",
                ".device.d",
                ".network.d",
                ".netdev.d",
                ".link.d",
                ".nspawn.d",
                ".container.d",
                ".zone.d",
                ".policy.d",
                ".rules.d",
                ".preset.d",
                ".wants.d",
                ".requires.d",
                ".requisite.d",
                ".conflicts.d",
                ".before.d",
                ".after.d",
                ".onboot.d",
                ".onfailure.d",
                ".onabort.d",
                ".onerror.d",
                ".onsuccess.d",
                ".onfinal.d",
                ".onrestart.d",
                ".onstop.d",
                ".onstart.d",
                ".onreload.d",
            }

            _, ext = os.path.splitext(file_path.lower())
            if ext in text_extensions:
                return True

            # 检查没有后缀名的常见文件
            filename = os.path.basename(file_path).lower()
            text_files_no_ext = {
                "readme",
                "license",
                "changelog",
                "authors",
                "contributors",
                "copying",
                "install",
                "news",
                "todo",
                "version",
                "manifest",
                "makefile",
                "dockerfile",
                "vagrantfile",
                "gemfile",
                "rakefile",
                "procfile",
                "pipfile",
                "poetry.lock",
                "requirements",
                "setup.cfg",
                "tox.ini",
                "pytest.ini",
                "mypy.ini",
                "flake8",
                "pylintrc",
                ".gitignore",
                ".gitattributes",
                ".dockerignore",
                ".eslintignore",
                ".prettierignore",
                ".stylelintignore",
                ".editorconfig",
            }

            if filename in text_files_no_ext:
                return True

            # 尝试读取文件前几个字节来判断是否为文本
            try:
                with open(file_path, "rb") as f:
                    chunk = f.read(1024)  # 读取前1024字节
                    if b"\x00" in chunk:  # 包含null字节，可能是二进制文件
                        return False
                    # 尝试解码为UTF-8
                    chunk.decode("utf-8")
                    return True
            except (UnicodeDecodeError, IOError, PermissionError):
                return False

        except Exception as e:
            logger.debug(f"判断文件类型时出错 {file_path}: {e}")
            return False

    def read_file_content(self, file_path):
        return FileUtils.read_file(file_path)

    def convert_to_source_code(self, file_path):
        module_name = file_path
        try:
            source_code = self.read_file_content(file_path)
        except Exception as e:
            logger.warning(f"Failed to read file: {file_path}. Error: {str(e)}")
            return None
        return SourceCode(module_name=module_name, source_code=source_code)

    def get_source_codes(self) -> Generator[SourceCode, None, None]:
        """遍历目录收集源代码文件，使用ignorefiles模块进行过滤"""
        for root, dirs, files in os.walk(self.directory, followlinks=True):
            # 过滤目录，移除应该被忽略的目录
            dirs[:] = [
                d
                for d in dirs
                if not should_ignore(os.path.join(root, d), self.directory)
            ]

            for file in files:
                file_path = os.path.join(root, file)

                # 跳过 .autocoderignore 文件本身
                if file == ".autocoderignore":
                    continue

                # 使用ignorefiles模块检查是否应该忽略此文件
                if should_ignore(file_path, self.directory):                    
                    continue

                # 检查是否为文本文件
                if not self.is_text_file(file_path):                    
                    continue

                # 应用自定义文件过滤器（如果有）
                if self.file_filter is None or self.file_filter(file_path, []):
                    source_code = self.convert_to_source_code(file_path)
                    if source_code is not None:
                        yield source_code

    def get_rest_source_codes(self) -> Generator[SourceCode, None, None]:
        if self.args.urls:
            urls = self.args.urls
            if isinstance(self.args.urls, str):
                urls = self.args.urls.split(",")
            http_doc = HttpDoc(args=self.args, llm=self.llm, urls=urls)
            sources = http_doc.crawl_urls()
            for source in sources:
                source.tag = "REST"
            return sources
        return []

    def get_rag_source_codes(self):
        if not self.args.enable_rag_search and not self.args.enable_rag_context:
            return []

        console = Console()
        console.print(
            f"\n[bold blue]Starting RAG search for:[/bold blue] {self.args.query}"
        )

        from autocoder.rag.rag_entry import RAGFactory

        rag = RAGFactory.get_rag(self.llm, self.args, "")
        docs = rag.search(self.args.query)
        for doc in docs:
            doc.tag = "RAG"

        console = Console()
        console.print(f"[bold green]Found {len(docs)} relevant documents[/bold green]")

        return docs

    def get_search_source_codes(self):
        temp = self.get_rag_source_codes()
        if self.args.search_engine and self.args.search_engine_token:
            if self.args.search_engine == "bing":
                search_engine = SearchEngine.BING
            else:
                search_engine = SearchEngine.GOOGLE

            searcher = Search(
                args=self.args,
                llm=self.llm,
                search_engine=search_engine,
                subscription_key=self.args.search_engine_token,
            )
            search_query = self.args.search or self.args.query
            search_context = searcher.answer_with_the_most_related_context(search_query)
            return temp + [
                SourceCode(
                    module_name="SEARCH_ENGINE",
                    source_code=search_context,
                    tag="SEARCH",
                )
            ]
        return temp + []

    @byzerllm.prompt()
    def get_simple_directory_structure(self) -> str:
        """
        当前项目目录结构：
        1. 项目根目录： {{ directory }}
        2. 项目子目录/文件列表：
        {{ structure }}
        """
        structure = []
        for source_code in self.get_source_codes():
            relative_path = os.path.relpath(source_code.module_name, self.directory)
            structure.append(relative_path)

        subs = "\n".join(sorted(structure))
        return {"directory": self.directory, "structure": subs}

    @byzerllm.prompt()
    def get_tree_like_directory_structure(self) -> str:
        """
        当前项目目录结构：
        1. 项目根目录： {{ directory }}
        2. 项目子目录/文件列表(类似tree 命令输出)：
        {{ structure }}
        """
        structure_dict = {}
        for source_code in self.get_source_codes():
            relative_path = os.path.relpath(source_code.module_name, self.directory)
            parts = relative_path.split(os.sep)
            current_level = structure_dict
            for part in parts:
                if part not in current_level:
                    current_level[part] = {}
                current_level = current_level[part]

        def generate_tree(d, indent=""):
            tree = []
            for k, v in d.items():
                if v:
                    tree.append(f"{indent}{k}/")
                    tree.extend(generate_tree(v, indent + "    "))
                else:
                    tree.append(f"{indent}{k}")
            return tree

        return {
            "structure": "\n".join(generate_tree(structure_dict)),
            "directory": self.directory,
        }

    def run(self):
        if self.git_url is not None:
            self.clone_repository()

        # 先收集所有sources
        for code in self.get_source_codes():
            self.sources.append(code)

        for code in self.get_rest_source_codes():
            self.sources.append(code)

        for code in self.get_search_source_codes():
            self.sources.append(code)

        # 如果target_file存在，才写入文件
        if self.target_file:
            with open(self.target_file, "w", encoding="utf-8") as file:
                for code in self.sources:
                    file.write(f"##File: {code.module_name}\n")
                    file.write(f"{code.source_code}\n\n")

    def clone_repository(self):
        if self.git_url is None:
            raise ValueError("git_url is required to clone the repository")

        if os.path.exists(self.directory):
            print(f"Directory {self.directory} already exists. Skipping cloning.")
        else:
            print(f"Cloning repository {self.git_url} into {self.directory}")
            Repo.clone_from(self.git_url, self.directory)



import os
import time
import json
import pyautogui
import numpy as np
from PIL import Image, ImageDraw, ImageFont
import byzerllm
from loguru import logger
from typing import List, Dict, Tuple, Optional, Any, Union
import platform
from autocoder.common import AutoCoderArgs
from byzerllm.utils.client import code_utils
from rich.console import Console
from rich.panel import Panel
from rich.text import Text
import pydantic

class LoadingStatus(pydantic.BaseModel):
    should_wait: bool
    reason: str
class ComputerUse:
    """
    计算机交互工具类，提供以下功能：
    1. 屏幕截图
    2. 检测屏幕上的物体
    3. 鼠标点击
    4. 鼠标滑动
    5. 键盘输入
    """
    
    def __init__(
        self,
        llm: byzerllm.ByzerLLM,
        args: AutoCoderArgs,
        screenshot_dir: str = None,
        pause: float = 0.5  # 每次操作后的暂停时间
    ):
        """
        初始化计算机交互工具
        
        Args:
            llm: ByzerLLM 实例，用于视觉分析
            args: AutoCoderArgs 配置
            screenshot_dir: 截图保存目录，如果为 None 则使用 args.output
            pause: 每次操作后的暂停时间，单位为秒
        """
        self.llm = llm
        # 获取视觉模型子客户端
        if llm.get_sub_client("vl_model"):
            self.vl_model = llm.get_sub_client("vl_model")
        else:
            self.vl_model = self.llm
            
        self.args = args
        self.screenshot_dir = screenshot_dir or os.path.join(args.output, "screenshots")
        os.makedirs(self.screenshot_dir, exist_ok=True)
        
        # 设置 pyautogui 暂停时间
        pyautogui.PAUSE = pause
        
        # 获取系统信息
        self.system = platform.system()
        logger.info(f"运行于 {self.system} 系统")
        
        # 获取屏幕尺寸
        self.screen_width, self.screen_height = pyautogui.size()
        logger.info(f"屏幕尺寸: {self.screen_width}x{self.screen_height}")
        
        # 设置 pyautogui 的失败安全机制
        pyautogui.FAILSAFE = True  # 将鼠标移动到屏幕左上角会触发异常，中断程序
        
    def screenshot(self, filename: str = None) -> str:
        """
        截取屏幕并保存为图片
        
        Args:
            filename: 保存的文件名，如果为 None 则自动生成
            
        Returns:
            截图的保存路径
        """
        # 生成文件名
        if filename is None:
            timestamp = time.strftime("%Y%m%d_%H%M%S", time.localtime())
            filename = f"screenshot_{timestamp}.png"
        
        # 确保文件名有扩展名
        if not filename.lower().endswith(('.png', '.jpg', '.jpeg')):
            filename += '.png'
            
        # 完整路径
        filepath = os.path.join(self.screenshot_dir, filename)
        
        # 截屏
        screenshot = pyautogui.screenshot()
        screenshot.save(filepath)
        logger.info(f"屏幕截图已保存到 {filepath}")
        
        return filepath

    @byzerllm.prompt()
    def detect_objects(self, image_path: str) -> str:
        """
        {{ image }}
        请分析这张屏幕截图，识别出其中的各种界面元素(如按钮、输入框、菜单、图标等)，并给出每个元素的bounding box坐标。
        
        bouding box 使用 (xmin, ymin, xmax, ymax) 来表示，其中xmin, ymin: 表示矩形左上角的坐标
        xmax, ymax: 表示矩形右下角的坐标
        
        最后按如下格式返回：
        ```json
        {
            "objects": [
                {
                    "type": "button",  # 元素类型，如 button, input, icon, menu, text 等
                    "bounding_box": [xmin, ymin, xmax, ymax],
                    "text": "按钮上的文字或元素描述",
                    "confidence": 0.95  # 可选，置信度
                },
                ...
            ]
        }
        ```
        """
        image = byzerllm.Image.load_image_from_path(image_path)
        return {"image": image}
    
    @byzerllm.prompt()
    def find_elements(self, image_path: str, element_desc: str) -> str:
        """
        {{ image }}
    
        请在屏幕截图中找到所有符合以下描述的用户界面元素:
        <element_desc>
        {{ element_desc }}
        </element_desc>
        
        请考虑各种类型的UI元素，包括但不限于:
        - 按钮(button)
        - 输入框(input)
        - 下拉菜单(dropdown)
        - 复选框(checkbox)
        - 单选按钮(radio)
        - 标签(label)
        - 图标(icon)
        - 菜单项(menu item)
        - 导航栏(navigation)
        - 滑块(slider)
        
        请注意以下几点:
        1. 即使元素仅部分匹配描述，也请返回
        2. 对于文本元素，请尝试识别包含相似或相关文本的元素
        3. 考虑元素在界面中的上下文和位置关系
        4. 适应不同的视觉样式(包括深色/浅色主题)
        5. 如果找到多个匹配项，请全部返回并按置信度排序
        
        对于每个找到的元素，请提供以下信息:
        - bounding box坐标 (xmin, ymin, xmax, ymax),其中 xmin,ymin表示左上角坐标，xmax,ymax表示右下角坐标。
        - 元素类型(如按钮、输入框等)
        - 元素上的文本或描述
        - 匹配的置信度(0-1之间)
        
        最后按如下JSON格式返回:
        ```json
        {               
            "objects": [
                {
                    "type": "button",  // 元素类型
                    "bounding_box": [xmin, ymin, xmax, ymax],
                    "text": "元素上的文字或描述",
                    "confidence": 0.95  // 置信度，值越高表示匹配越确定
                },
                // 更多元素...
            ]
        }
        ```
        
        如果完全没有找到匹配的元素，请返回空objects数组:
        ```json
        {
            "objects": []
        }
        """
        image = byzerllm.Image.load_image_from_path(image_path)
        return {"image": image, "element_desc": element_desc}
        
    def click(self, x: int, y: int, button: str = 'left', clicks: int = 1) -> None:
        """
        在指定位置执行鼠标点击
        
        Args:
            x: 横坐标
            y: 纵坐标
            button: 鼠标按钮，可选 'left', 'right', 'middle'
            clicks: 点击次数
        """
        # 确保坐标在屏幕范围内
        x = max(0, min(x, self.screen_width - 1))
        y = max(0, min(y, self.screen_height - 1))
        
        logger.info(f"鼠标点击: 坐标({x}, {y}), 按钮: {button}, 次数: {clicks}")
        pyautogui.click(x=x, y=y, button=button, clicks=clicks)
        
    def draw_bounding_box(self, image_path: str, bbox: List[int], element_desc: str, 
                         center_point: Optional[Tuple[int, int]] = None, 
                         confidence: Optional[float] = None) -> str:
        """
        在图像上绘制边界框、标签和中心点
        
        Args:
            image_path: 原始图像路径
            bbox: 边界框坐标 [x1, y1, x2, y2]
            element_desc: 元素描述
            center_point: 可选，中心点坐标 (x, y)
            confidence: 可选，置信度
            
        Returns:
            保存的带边界框图像路径
        """
        try:
            # 创建临时目录存储带框图片
            bbox_dir = os.path.join(self.screenshot_dir, "bboxes")
            os.makedirs(bbox_dir, exist_ok=True)
            
            # 生成新文件名
            timestamp = time.strftime("%Y%m%d_%H%M%S", time.localtime())
            bbox_image_path = os.path.join(bbox_dir, f"bbox_{timestamp}.png")
            
            # 加载原始图片
            img = Image.open(image_path)
            draw = ImageDraw.Draw(img)
            
            # 绘制矩形框
            draw.rectangle(
                [(bbox[0], bbox[1]), (bbox[2], bbox[3])],
                outline="red",
                width=3
            )
            
            # 如果提供了中心点，绘制中心点
            if center_point:
                cx, cy = center_point
                draw.ellipse(
                    [(cx-5, cy-5), (cx+5, cy+5)],
                    fill="blue"
                )
            
            # 准备标签文本
            if confidence is not None and isinstance(confidence, (int, float)):
                label_text = f"{element_desc} ({confidence:.2f})"
            else:
                label_text = element_desc
            
            # 设置文本位置
            text_x = bbox[0]
            text_y = max(0, bbox[1] - 20)  # 文本位于框上方
            
            # 绘制文本背景
            text_width = len(label_text) * 8  # 估算文本宽度
            draw.rectangle(
                [(text_x, text_y), (text_x + text_width, text_y + 20)],
                fill="yellow"
            )
            
            # 绘制文本
            draw.text(
                (text_x, text_y),
                label_text,
                fill="black"
            )
            
            # 在图片上添加时间戳和文件信息
            footer_text = f"Time: {time.strftime('%Y-%m-%d %H:%M:%S')} | File: {os.path.basename(image_path)}"
            img_width, img_height = img.size
            footer_y = img_height - 20
            
            # 添加底部信息栏
            draw.rectangle(
                [(0, footer_y - 5), (img_width, img_height)],
                fill="black"
            )
            draw.text(
                (10, footer_y),
                footer_text,
                fill="white"
            )
            
            # 保存图片
            img.save(bbox_image_path)
            logger.info(f"已保存带边界框的图片: {bbox_image_path}")
            print(f"识别到元素 '{element_desc}' - 已保存边界框图片: {bbox_image_path}")
            
            return bbox_image_path
            
        except Exception as e:
            logger.error(f"绘制边界框时出错: {str(e)}")
            return None
    
    
    def click_element(self, screenshot_path: str, element_desc: str) -> bool:
        """
        查找并点击指定描述的元素
        
        Args:
            screenshot_path: 屏幕截图路径
            element_desc: 元素描述
            
        Returns:
            是否成功点击
        """
        # 查找元素
        response = self.find_elements.with_llm(self.vl_model).run(screenshot_path, element_desc)
        logger.info(f"查找元素结果: {response}")
        
        try:
            # 解析JSON结果
            result_json = code_utils.extract_code(response)[-1][1]
            result = json.loads(result_json)
            objects = result.get("objects", [])
            sorted_objects = sorted(objects, key=lambda x: x.get("confidence", 0), reverse=True)
                        
            for element in sorted_objects:
                bbox = element.get("bounding_box", [])
                
                if len(bbox) == 4:
                    # 计算中心点
                    center_x = int((bbox[0] + bbox[2]) / 2)
                    center_y = int((bbox[1] + bbox[3]) / 2)
                    
                    # 绘制边界框并保存图像
                    element_text = element.get('text', element_desc)
                    confidence = element.get('confidence', None)
                    
                    self.draw_bounding_box(
                        image_path=screenshot_path,
                        bbox=bbox,
                        element_desc=element_text,
                        center_point=(center_x, center_y),
                        confidence=confidence
                    )
                    
                    # 点击中心点
                    self.click(center_x, center_y)
                    logger.info(f"点击元素: {element_text}")
                    return True
            
            logger.warning(f"未找到元素: {element_desc}")
            return False
        
        except Exception as e:
            logger.error(f"解析查找结果时出错: {str(e)}")
            return False
    
    def drag(self, start_x: int, start_y: int, end_x: int, end_y: int, duration: float = 0.5) -> None:
        """
        从一个位置拖动到另一个位置
        
        Args:
            start_x: 起始位置X坐标
            start_y: 起始位置Y坐标
            end_x: 结束位置X坐标
            end_y: 结束位置Y坐标
            duration: 拖动持续时间，单位为秒
        """
        # 确保坐标在屏幕范围内
        start_x = max(0, min(start_x, self.screen_width - 1))
        start_y = max(0, min(start_y, self.screen_height - 1))
        end_x = max(0, min(end_x, self.screen_width - 1))
        end_y = max(0, min(end_y, self.screen_height - 1))
        
        logger.info(f"鼠标拖动: 从({start_x}, {start_y})到({end_x}, {end_y}), 持续时间: {duration}秒")
        
        # 移动到起始位置
        pyautogui.moveTo(start_x, start_y)
        # 拖动到目标位置
        pyautogui.dragTo(end_x, end_y, duration=duration)
        
    def scroll(self, x: int, y: int, clicks: int) -> None:
        """
        在指定位置滚动鼠标滚轮
        
        Args:
            x: 横坐标
            y: 纵坐标
            clicks: 滚动量，正数向上滚动，负数向下滚动
        """
        # 确保坐标在屏幕范围内
        x = max(0, min(x, self.screen_width - 1))
        y = max(0, min(y, self.screen_height - 1))
        
        direction = "上" if clicks > 0 else "下"
        logger.info(f"鼠标滚动: 坐标({x}, {y}), 方向: {direction}, 量: {abs(clicks)}")
        
        # 移动到指定位置
        pyautogui.moveTo(x, y)
        # 滚动
        pyautogui.scroll(clicks)
        
    def type_text(self, text: str, interval: float = 0.05) -> None:
        """
        模拟键盘输入文本
        
        Args:
            text: 要输入的文本
            interval: 按键间隔时间，单位为秒
        """
        logger.info(f"键盘输入: '{text}'")
        pyautogui.typewrite(text, interval=interval)
        
    def press_key(self, key):
        """
        按下指定的键或键组合
        
        Args:
            key: 键名或键组合列表，如'enter'或['ctrl', 'c']
        """
        try:
            if isinstance(key, list):
                # 处理组合键
                logger.info(f"按下组合键: {'+'.join(key)}")
                pyautogui.hotkey(*key)  # 使用hotkey方法处理组合键
            else:
                # 处理单个键
                logger.info(f"按下按键: {key}")
                pyautogui.press(key)
        except Exception as e:
            logger.error(f"按键操作失败: {str(e)}")
            raise
            
    def extract_text(self, screenshot_path: str, region: List[int] = None) -> str:
        """
        从屏幕截图中提取文本
        
        Args:
            screenshot_path: 屏幕截图路径
            region: 区域坐标 [xmin, ymin, xmax, ymax]，如果为None则处理整个截图
            
        Returns:
            提取的文本
        """
        if region is not None:
            # 裁剪指定区域
            img = Image.open(screenshot_path)
            cropped = img.crop(region)
            
            # 保存临时文件
            temp_path = os.path.join(self.screenshot_dir, "temp_region.png")
            cropped.save(temp_path)
            screenshot_path = temp_path
            
        @byzerllm.prompt()
        def extract_text_from_image(image_path: str) -> str:
            """
            {{ image }}
            请提取这张图片中的所有文本内容，保持原有布局和顺序。
            """
            image = byzerllm.Image.load_image_from_path(image_path)
            return {"image": image}
            
        # 调用VL模型提取文本
        result = extract_text_from_image.with_llm(self.vl_model).run(screenshot_path)
        return result
    
    def run_workflow(self, steps: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        """
        执行一系列操作步骤
        
        Args:
            steps: 操作步骤列表，每个步骤是一个字典，包含 'action' 和相关参数
                  支持的 action: screenshot, detect, click, find_and_click, type, press, 
                  drag, scroll, extract_text, ask_user, response_user, open_browser, should_wait_loading
            
        Returns:
            执行结果列表
        """
        results = []
        
        for i, step in enumerate(steps):
            try:
                action = step.get('action', '')
                logger.info(f"执行步骤 {i+1}: {action}")
                
                result = {'step': i+1, 'action': action, 'success': True}
                
                if action == 'screenshot':
                    filename = step.get('filename')
                    path = self.screenshot(filename)
                    result['path'] = path
                    result["should_verify"] = False

                elif action == 'focus_app':
                    app_name = step.get('app_name')
                    retry_count = step.get('retry_count', 3)
                    success = self.focus_app(app_name, retry_count)
                    result['success'] = success
                    result['app_name'] = app_name
                    
                elif action == 'detect':
                    image_path = step.get('image_path')
                    if not image_path:
                        # 自动截图
                        image_path = self.screenshot(f"auto_step_{i+1}.png")
                    
                    detection = self.detect_objects.with_llm(self.vl_model).run(image_path)
                    result_json = code_utils.extract_code(detection)[-1][1]
                    result['objects'] = json.loads(result_json)
                    
                elif action == 'click':
                    x = step.get('x')
                    y = step.get('y')
                    button = step.get('button', 'left')
                    clicks = step.get('clicks', 1)
                    
                    self.click(x, y, button, clicks)
                    result['coordinates'] = [x, y]
                    
                elif action == 'find_and_click':
                    element_desc = step.get('element_desc')
                    image_path = step.get('image_path')
                    if not image_path:
                        # 自动截图
                        image_path = self.screenshot(f"auto_step_{i+1}.png")
                        
                    success = self.click_element(image_path, element_desc)
                    result['success'] = success
                    result['element_desc'] = element_desc
                    
                elif action == 'drag':
                    start_x = step.get('start_x')
                    start_y = step.get('start_y')
                    end_x = step.get('end_x')
                    end_y = step.get('end_y')
                    duration = step.get('duration', 0.5)
                    
                    self.drag(start_x, start_y, end_x, end_y, duration)
                    result['from'] = [start_x, start_y]
                    result['to'] = [end_x, end_y]
                    
                elif action == 'type':
                    text = step.get('text', '')
                    interval = step.get('interval', 0.05)
                    
                    self.type_text(text, interval)
                    result['text'] = text
                    
                elif action == 'press':
                    key = step.get('key')
                    self.press_key(key)
                    result['key'] = key
                    
                elif action == 'scroll':
                    clicks = step.get('clicks')
                    x = step.get('x', None)
                    y = step.get('y', None)
                    
                    # 如果没有提供坐标，使用当前鼠标位置
                    current_pos = pyautogui.position()
                    if x is None:
                        x = current_pos.x
                    if y is None:
                        y = current_pos.y
                        
                    self.scroll(x, y, clicks)
                    result['coordinates'] = [x, y, clicks]
                    
                elif action == 'extract_text':
                    image_path = step.get('image_path')
                    region = step.get('region')
                    
                    if not image_path:
                        # 自动截图
                        image_path = self.screenshot(f"auto_step_{i+1}.png")
                        
                    text = self.extract_text(image_path, region)
                    result['text'] = text
                
                elif action == 'ask_user':                    
                    logger.info(f"需要询问用户: {question}")
                    question = step.get('question', '')
                    response = self.ask_user(question)
                    result['question'] = question
                    result['action_required'] = 'ask_user'
                    result['response'] = response
                    
                
                elif action == 'response_user':
                    logger.info(f"需要向用户显示: {response}")
                    response = step.get('response', '')
                    self.response_user(response)
                    result['response'] = response
                    result['action_required'] = 'response_user'                    
                
                elif action == 'open_browser':
                    browser_name = step.get('browser_name', 'chrome')                                        
                    browser_result = self.open_browser(browser_name)
                    result.update(browser_result)
                
                elif action == "wait_loading":
                    target = step.get('target')                                                          
                    
                    # 调用_should_wait_loading方法分析是否需要等待加载
                    wait_analysis = self.should_wait_loading(
                        target=target                        
                    )
                    
                    # 如果需要等待，自动等待一段时间
                    wait_times = 0
                    wait_time = 10

                    while wait_analysis.should_wait and wait_times < 10:
                        wait_times += 1
                        logger.info(f"系统正在加载中，自动等待 {wait_time} 秒...")
                        time.sleep(wait_time)
                        wait_analysis = self.should_wait_loading(
                            target=target                        
                        )
                        result['should_wait'] = wait_analysis.should_wait 
                        result['reason'] = wait_analysis.reason
                    result['should_wait'] = wait_analysis.should_wait
                    result['reason'] = wait_analysis.reason
                    result["should_verify"] = False
                        
                
                else:
                    logger.warning(f"未知操作: {action}")
                    result['success'] = False
                    result['error'] = f"未知操作: {action}"
                
                # 添加执行结果
                results.append(result)
                
                # 如果设置了等待时间
                if 'wait' in step:
                    wait_time = float(step['wait'])
                    logger.info(f"等待 {wait_time} 秒")
                    time.sleep(wait_time)
                    
            except Exception as e:
                logger.error(f"执行步骤 {i+1} 时出错: {str(e)}")
                results.append({
                    'step': i+1, 
                    'action': step.get('action', 'unknown'),
                    'success': False,
                    'error': str(e)
                })
                
                # 如果设置了出错继续
                if not step.get('continue_on_error', False):
                    break
                    
        return results
    

    def ask_user(self, question: str) -> str:
        """
        向用户提问，获取用户输入
        
        Args:
            question: 问题内容
            
        Returns:
            用户的回答
        """
        console = Console()
        
        # 创建一个醒目的问题面板
        question_text = Text(question, style="bold cyan")
        question_panel = Panel(
            question_text,
            title="[bold yellow]Web Automation Question[/bold yellow]",
            border_style="blue",
            expand=False
        )
        
        # 显示问题面板
        console.print(question_panel)
        
        # 获取用户输入
        try:
            from prompt_toolkit import PromptSession
            session = PromptSession(message=self.printer.get_message_from_key('web_automation_ask_user', default="Your answer: "))
            answer = session.prompt()
        except (ImportError, KeyboardInterrupt):
            # 降级到标准输入或处理中断
            answer = input("Your answer: ")
        
        # 记录交互
        self.save_to_memory_file(
            query=question,
            response=answer
        )
        
        return answer

    def response_user(self, response: str) -> str:
        """
        直接向用户显示消息，无需等待用户输入
        
        Args:
            response: 要显示的消息内容
            
        Returns:
            显示的消息内容
        """
        console = Console()
        
        # 创建一个醒目的消息面板
        message_text = Text(response, style="italic")
        message_panel = Panel(
            message_text,
            title="",
            border_style="green",
            expand=False
        )
        
        # 显示消息面板
        console.print(message_panel)
        
        # 记录交互
        self.save_to_memory_file(
            query="system_message",
            response=response
        )
        
        return response
            
    def open_browser(self, browser_name="chrome", url=None):
        """
        打开指定的浏览器并可选择导航到URL。比直接使用系统命令更可靠的浏览器启动方式。
        
        Args:
            browser_name: 浏览器名称，默认为"chrome"，也支持"firefox"和"edge"等
            url: 可选参数，启动后要导航到的网址
            
        Returns:
            操作结果字典
        """
        import subprocess
        import platform
        import time
        
        system = platform.system()
        result = {"success": False, "message": ""}
        
        try:
            # 根据浏览器名称和操作系统打开浏览器
            if browser_name.lower() == "chrome":
                if system == 'Windows':
                    subprocess.Popen(['start', 'chrome'], shell=True)
                elif system == 'Darwin':  # macOS
                    subprocess.Popen(['open', '-a', 'Google Chrome'])
                elif system == 'Linux':
                    subprocess.Popen(['google-chrome'])
            elif browser_name.lower() == "firefox":
                if system == 'Windows':
                    subprocess.Popen(['start', 'firefox'], shell=True)
                elif system == 'Darwin':  # macOS
                    subprocess.Popen(['open', '-a', 'Firefox'])
                elif system == 'Linux':
                    subprocess.Popen(['firefox'])
            elif browser_name.lower() == "edge":
                if system == 'Windows':
                    subprocess.Popen(['start', 'msedge'], shell=True)
                elif system == 'Darwin':  # macOS
                    subprocess.Popen(['open', '-a', 'Microsoft Edge'])
                elif system == 'Linux':
                    subprocess.Popen(['microsoft-edge'])  

            result["success"] = True
            return result
        
        except Exception as e:
            result["message"] = f"启动{browser_name}浏览器时出错: {str(e)}"
            logger.error(f"启动{browser_name}浏览器时出错: {str(e)}")
            return result

    def element_exists(self, screenshot_path: str, element_desc: str) -> bool:
        """
        检查屏幕上是否存在指定描述的元素
        
        Args:
            screenshot_path: 屏幕截图路径
            element_desc: 元素描述
            
        Returns:
            元素是否存在
        """
        # 查找元素
        response = self.find_elements.with_llm(self.vl_model).run(screenshot_path, element_desc)
        logger.info(f"检查元素是否存在: {element_desc}")
        
        try:
            # 解析JSON结果
            result_json = code_utils.extract_code(response)[-1][1]
            result = json.loads(result_json)
            objects = result.get("objects", [])
            
            # 如果objects列表不为空，表示找到了元素
            exists = len(objects) > 0
            
            if exists:
                # 找到了元素，提供第一个匹配元素的信息
                best_match = sorted(objects, key=lambda x: x.get("confidence", 0), reverse=True)[0]
                confidence = best_match.get("confidence", 0)
                text = best_match.get("text", "")
                
                # 可选：绘制边界框并保存图片
                bbox = best_match.get("bounding_box", [])
                if len(bbox) == 4:
                    try:
                        # 计算中心点坐标
                        center_x = int((bbox[0] + bbox[2]) / 2)
                        center_y = int((bbox[1] + bbox[3]) / 2)
                        
                        # 绘制边界框
                        self.draw_bounding_box(
                            image_path=screenshot_path,
                            bbox=bbox,
                            element_desc=text or element_desc,
                            center_point=(center_x, center_y),
                            confidence=confidence
                        )
                    except Exception as e:
                        logger.error(f"绘制边界框时出错: {str(e)}")
                
                logger.info(f"找到元素 '{element_desc}', 置信度: {confidence}, 文本: '{text}'")
            else:
                logger.info(f"未找到元素 '{element_desc}'")
            
            return exists
            
        except Exception as e:
            logger.error(f"检查元素是否存在时出错: {str(e)}")
            return False
        
    @byzerllm.prompt()
    def _should_wait_loading(self, target: str) ->str:
        """        
        {{ current_screenshot }}
        
        考虑到诸如点击链接等动作，点击后不会立马发生屏幕变化，而用户期待看到的内容为：

        {{ target }}
        
        
        我们需要判断用户期待的内容是否已经出现。如果系统还处于加载状态，并且用户期待的内容还没有出现，请返回如下JSON格式：
        ```json
        {
            "should_wait": true,
            "reason": "判断是否处于加载状态分析理由"
        }
        ```

        如果用户期待的内容已经出现，请返回如下JSON格式：
        ```json
        {
            "should_wait": false,
            "reason": "判断是否处于加载状态分析理由"    
        }
        ```
        """  
        current_screenshot_path = self.screenshot(f"current_screenshot_{int(time.time())}.png")
        current_screenshot = byzerllm.Image.load_image_from_path(current_screenshot_path)
        return {
            "current_screenshot":current_screenshot,            
        }


    def should_wait_loading(self, target: str) -> LoadingStatus:
        """
        判断是否需要等待加载完成
        """
        response = self._should_wait_loading.with_llm(self.vl_model).with_return_type(LoadingStatus).run(target)        
        return response

    def focus_app(self, app_name: str, retry_count: int = 3) -> bool:
        """
        查找并聚焦指定的应用程序窗口
        
        Args:
            app_name: 应用程序名称或窗口标题的一部分
            retry_count: 重试次数，默认为3
            
        Returns:
            是否成功聚焦应用
        """
        logger.info(f"尝试聚焦应用: {app_name}")
        
        for attempt in range(retry_count):
            # 截取当前屏幕
            screenshot_path = self.screenshot(f"focus_app_{int(time.time())}.png")                        
                                                            
            # 查找元素并获取位置
            response = self.find_elements.with_llm(self.vl_model).run(screenshot_path, f"应用 {app_name},是否在屏幕上，比如标题栏，任务栏亦或者当前活动窗口")            
            try:
                result_json = code_utils.extract_code(response)[-1][1]
                result = json.loads(result_json)
                objects = result.get("objects", [])
                
                if objects:
                    # 找到匹配度最高的元素
                    best_match = sorted(objects, key=lambda x: x.get("confidence", 0), reverse=True)[0]
                    bbox = best_match.get("bounding_box", [])
                    
                    if len(bbox) == 4:
                        # 计算中心点
                        center_x = int((bbox[0] + bbox[2]) / 2)
                        center_y = int((bbox[1] + bbox[3]) / 2)
                        
                        # 点击应用窗口来聚焦
                        self.click(center_x, center_y)
                        logger.info(f"已点击 {app_name} 窗口，尝试聚焦")
                        
                        # 等待窗口获得焦点
                        time.sleep(0.5)
                        
                        # 再次截图以验证是否聚焦成功
                        verification_screenshot = self.screenshot(f"focus_app_verification_{int(time.time())}.png")
                        if self.element_exists(verification_screenshot, f"应用 {app_name} 是否在当前桌面"):
                            logger.info(f"成功聚焦应用: {app_name}")
                            return True
            except Exception as e:
                logger.error(f"聚焦应用时出错: {str(e)}")
            
            # 如果所有尝试都失败，等待一段时间后重试
            if attempt < retry_count - 1:
                logger.warning(f"聚焦 {app_name} 失败，将在1秒后重试 ({attempt+1}/{retry_count})")
                time.sleep(1)
        
        logger.error(f"无法聚焦应用: {app_name}，已达到最大重试次数")
        return False


"""
Tests for CommandExecutor functionality.

This module tests the main command execution interface with timeout
support and process management.
"""

import pytest
import time
import platform
from unittest.mock import patch, MagicMock

from ..command_executor import CommandExecutor, execute_command
from ..timeout_config import TimeoutConfig
from ..exceptions import CommandTimeoutError, CommandExecutionError


class TestCommandExecutor:
    """Test cases for CommandExecutor class."""
    
    def setup_method(self):
        """Set up test fixtures."""
        self.config = TimeoutConfig(
            default_timeout=5.0,
            cleanup_timeout=2.0,
            grace_period=1.0
        )
        self.executor = CommandExecutor(self.config, verbose=False)
    
    def teardown_method(self):
        """Clean up after tests."""
        self.executor.cleanup()
    
    def test_simple_command_execution(self):
        """Test basic command execution."""
        if platform.system() == "Windows":
            command = "echo Hello World"
        else:
            command = "echo 'Hello World'"
        
        exit_code, output = self.executor.execute(command)
        
        assert exit_code == 0
        assert "Hello World" in output
    
    def test_command_with_exit_code(self):
        """Test command that returns non-zero exit code."""
        if platform.system() == "Windows":
            command = "exit 1"
        else:
            command = "exit 1"
        
        exit_code, output = self.executor.execute(command)
        
        assert exit_code == 1
    
    def test_command_timeout(self):
        """Test command timeout functionality."""
        if platform.system() == "Windows":
            command = "ping -n 10 127.0.0.1"  # Windows ping command
        else:
            command = "sleep 10"  # Unix sleep command
        
        with pytest.raises(CommandTimeoutError) as exc_info:
            self.executor.execute(command, timeout=1.0)
        
        assert exc_info.value.timeout == 1.0
        assert command in str(exc_info.value)
    
    def test_command_with_working_directory(self):
        """Test command execution with custom working directory."""
        import tempfile
        import os
        
        with tempfile.TemporaryDirectory() as temp_dir:
            if platform.system() == "Windows":
                command = "cd"
            else:
                command = "pwd"
            
            exit_code, output = self.executor.execute(command, cwd=temp_dir)
            
            assert exit_code == 0
            assert temp_dir in output.replace("\\", "/")  # Normalize path separators
    
    def test_command_with_environment_variables(self):
        """Test command execution with custom environment variables."""
        env = {"TEST_VAR": "test_value"}
        
        if platform.system() == "Windows":
            command = "echo %TEST_VAR%"
        else:
            command = "echo $TEST_VAR"
        
        exit_code, output = self.executor.execute(command, env=env)
        
        assert exit_code == 0
        assert "test_value" in output
    
    def test_executor_status(self):
        """Test executor status reporting."""
        status = self.executor.get_status()
        
        assert isinstance(status, dict)
        assert "config" in status
        assert "active_processes" in status
        assert "performance_summary" in status
        assert "recent_alerts" in status
    
    def test_executor_cleanup(self):
        """Test executor cleanup functionality."""
        # Start a long-running command
        if platform.system() == "Windows":
            command = "ping -n 100 127.0.0.1"
        else:
            command = "sleep 100"
        
        # Execute in background (this will be a real test in a threaded environment)
        # For now, just test that cleanup doesn't crash
        self.executor.cleanup()
        
        # Verify executor is still functional after cleanup
        simple_command = "echo test"
        exit_code, output = self.executor.execute(simple_command)
        assert exit_code == 0

    def test_execute_background_basic(self):
        """Test basic background command execution."""
        if platform.system() == "Windows":
            command = "ping -n 1 127.0.0.1"
        else:
            command = "sleep 0.1"
        
        # Execute command in background
        process_info = self.executor.execute_background(command)
        
        # Verify we get process information
        assert "pid" in process_info
        assert "command" in process_info
        assert "status" in process_info
        assert process_info["pid"] > 0
        assert process_info["status"] == "running"
        
        # Verify process appears in background processes
        bg_processes = self.executor.get_background_processes()
        assert process_info["pid"] in bg_processes
        
        # Wait a bit for process to complete
        time.sleep(0.2)
        
        # Clean up
        self.executor.cleanup_background_process(process_info["pid"])

    def test_execute_background_with_working_directory(self):
        """Test background execution with custom working directory."""
        import tempfile
        
        with tempfile.TemporaryDirectory() as temp_dir:
            if platform.system() == "Windows":
                command = "echo test"
            else:
                command = "echo test"
            
            process_info = self.executor.execute_background(command, cwd=temp_dir)
            
            assert "pid" in process_info
            assert process_info["pid"] > 0
            assert "working_directory" in process_info
            
            # Clean up
            time.sleep(0.1)  # Give process time to complete
            self.executor.cleanup_background_process(process_info["pid"])

    def test_get_background_processes(self):
        """Test getting background processes information."""
        if platform.system() == "Windows":
            command = "ping -n 1 127.0.0.1"
        else:
            command = "sleep 0.1"
        
        # Start a background process
        process_info = self.executor.execute_background(command)
        pid = process_info["pid"]
        
        # Get background processes
        bg_processes = self.executor.get_background_processes()
        
        assert pid in bg_processes
        assert bg_processes[pid]["command"] == command
        assert "status" in bg_processes[pid]
        
        # Clean up
        time.sleep(0.2)
        self.executor.cleanup_background_process(pid)

    def test_cleanup_background_process(self):
        """Test cleanup of specific background process."""
        if platform.system() == "Windows":
            command = "ping -n 5 127.0.0.1"
        else:
            command = "sleep 1"
        
        # Start a background process
        process_info = self.executor.execute_background(command)
        pid = process_info["pid"]
        
        # Verify process is running
        bg_processes = self.executor.get_background_processes()
        assert pid in bg_processes
        
        # Clean up the specific process
        success = self.executor.cleanup_background_process(pid)
        assert success is True or success is False  # Should return a boolean
        
        # Process should be removed from background processes
        time.sleep(0.1)
        bg_processes = self.executor.get_background_processes()
        # Process might still be there if cleanup is in progress, that's ok


class TestExecuteCommand:
    """Test cases for convenience execute_command function."""
    
    def test_convenience_function(self):
        """Test the convenience execute_command function."""
        if platform.system() == "Windows":
            command = "echo Test"
        else:
            command = "echo 'Test'"
        
        exit_code, output = execute_command(command)
        
        assert exit_code == 0
        assert "Test" in output
    
    def test_convenience_function_with_timeout(self):
        """Test convenience function with timeout."""
        if platform.system() == "Windows":
            command = "ping -n 10 127.0.0.1"
        else:
            command = "sleep 10"
        
        with pytest.raises(CommandTimeoutError):
            execute_command(command, timeout=1.0)


class TestExecuteCommandBackground:
    """Test cases for convenience execute_command_background function."""
    
    def test_convenience_background_function(self):
        """Test the convenience execute_command_background function."""
        from ..command_executor import execute_command_background, cleanup_background_process
        
        if platform.system() == "Windows":
            command = "ping -n 1 127.0.0.1"
        else:
            command = "sleep 0.1"
        
        process_info = execute_command_background(command)
        
        assert "pid" in process_info
        assert "command" in process_info
        assert "status" in process_info
        assert process_info["pid"] > 0
        assert process_info["status"] == "running"
        
        # Clean up
        time.sleep(0.2)
        cleanup_background_process(process_info["pid"])
    
    def test_get_background_processes_convenience(self):
        """Test the convenience get_background_processes function."""
        from ..command_executor import (
            execute_command_background, 
            get_background_processes, 
            cleanup_background_process
        )
        
        if platform.system() == "Windows":
            command = "ping -n 1 127.0.0.1"
        else:
            command = "sleep 0.1"
        
        # Start a background process
        process_info = execute_command_background(command)
        pid = process_info["pid"]
        
        # Get background processes
        bg_processes = get_background_processes()
        
        assert pid in bg_processes
        assert bg_processes[pid]["command"] == command
        
        # Clean up
        time.sleep(0.2)
        cleanup_background_process(pid)
    
    def test_get_background_process_info_convenience(self):
        """Test the convenience get_background_process_info function."""
        from ..command_executor import (
            execute_command_background, 
            get_background_process_info, 
            cleanup_background_process
        )
        
        if platform.system() == "Windows":
            command = "ping -n 1 127.0.0.1"
        else:
            command = "sleep 0.1"
        
        # Start a background process
        process_info = execute_command_background(command)
        pid = process_info["pid"]
        
        # Get specific process info
        info = get_background_process_info(pid)
        
        assert info is not None
        assert info["command"] == command
        
        # Clean up
        time.sleep(0.2)
        cleanup_background_process(pid)


class TestCommandExecutorConfiguration:
    """Test cases for CommandExecutor configuration."""
    
    def test_custom_timeout_config(self):
        """Test executor with custom timeout configuration."""
        config = TimeoutConfig(default_timeout=2.0)
        executor = CommandExecutor(config)
        
        # Test that config is applied
        assert executor.config.default_timeout == 2.0
        
        executor.cleanup()
    
    def test_command_specific_timeout(self):
        """Test command-specific timeout configuration."""
        config = TimeoutConfig()
        config.set_command_timeout("sleep", 1.0)
        
        executor = CommandExecutor(config)
        
        # Verify timeout is applied for sleep command
        timeout_val = config.get_timeout_for_command("sleep 5")
        assert timeout_val == 1.0
        
        executor.cleanup()


class TestErrorHandling:
    """Test cases for error handling and recovery."""
    
    def test_invalid_command(self):
        """Test handling of invalid commands."""
        executor = CommandExecutor(verbose=False)
        
        # This should not raise an exception but return non-zero exit code
        exit_code, output = executor.execute("nonexistent_command_12345")
        
        assert exit_code != 0
        
        executor.cleanup()
    
    def test_process_cleanup_on_error(self):
        """Test that processes are cleaned up when errors occur."""
        executor = CommandExecutor(verbose=False)
        
        try:
            if platform.system() == "Windows":
                command = "ping -n 10 127.0.0.1"
            else:
                command = "sleep 10"
            
            with pytest.raises(CommandTimeoutError):
                executor.execute(command, timeout=0.5)
        
        finally:
            # Verify cleanup works
            executor.cleanup()
            
            # Should have no active processes after cleanup
            status = executor.get_status()
            assert status["active_processes"] == 0


@pytest.mark.integration
class TestIntegrationScenarios:
    """Integration test scenarios."""
    
    def test_multiple_concurrent_commands(self):
        """Test handling of multiple commands (simulated)."""
        executor = CommandExecutor(verbose=False)
        
        try:
            # Execute multiple simple commands
            commands = [
                "echo 'Command 1'",
                "echo 'Command 2'", 
                "echo 'Command 3'"
            ]
            
            results = []
            for cmd in commands:
                if platform.system() == "Windows":
                    cmd = cmd.replace("'", '"')
                exit_code, output = executor.execute(cmd)
                results.append((exit_code, output))
            
            # Verify all commands succeeded
            for i, (exit_code, output) in enumerate(results):
                assert exit_code == 0
                assert f"Command {i+1}" in output
        
        finally:
            executor.cleanup()
    
    def test_timeout_and_recovery(self):
        """Test timeout handling and recovery."""
        executor = CommandExecutor(verbose=False)
        
        try:
            # Test timeout
            if platform.system() == "Windows":
                long_command = "ping -n 10 127.0.0.1"
            else:
                long_command = "sleep 5"
            
            with pytest.raises(CommandTimeoutError):
                executor.execute(long_command, timeout=1.0)
            
            # Test that executor still works after timeout
            simple_command = "echo 'Recovery test'"
            if platform.system() == "Windows":
                simple_command = 'echo "Recovery test"'
            
            exit_code, output = executor.execute(simple_command)
            assert exit_code == 0
            assert "Recovery test" in output
        
        finally:
            executor.cleanup()


if __name__ == "__main__":
    pytest.main([__file__, "-v"]) 